import ast,os
import tokenize



abs_path=os.path.abspath(os.path.dirname(__file__))
abs_path=abs_path[:abs_path.index("extract_transform_complicate_code_new")]
import sys
sys.path.append(abs_path)
sys.path.append(abs_path+"extract_transform_complicate_code_new/")
sys.path.append(abs_path+"extract_transform_complicate_code_new/comprehension/")
import complicated_code_util
import util
import extract_compli_for_comprehension_logic
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
from RefactoringIdioms.transform_c_s import transform_comprehension_dict_compli_to_simple
from tokenize import tokenize
from io import BytesIO
from pathos.multiprocessing import ProcessingPool as newPool
import traceback
'''
get comprehension node
Block=> Assign;...; For
For=>”for” ( For| If|If-else|body)
If=>”if” (body| For| If| If-else)
If-else=>”if” body “else” (body| If-else)
body=> (Assign|Call)*;Append*
并且里面只有一个写变量
for

    b.append()
    
    a.append(b)

'''

const_empty_list=[" = dict()"," = {}"]


def get_var(one_body):
    var_name=None
    if isinstance(one_body, ast.Assign):
        targets = one_body.targets
        if len(targets) > 1:
            return False

        left_node = targets[0]
        if isinstance(left_node, ast.Subscript):
            var_name=ast.unparse(left_node.value)
    return var_name



def whether_fun_is_append(one_body,assign_block_list):
    #print("one_body: ",ast.unparse(one_body),one_body.__dict__)
    if isinstance(one_body, ast.Assign):
        #print("come here")
        targets=one_body.targets
        if len(targets)>1:
            return False
        left_node=targets[0]
        if isinstance(left_node, ast.Subscript):
            assign_block_list.append([one_body])
            return True
    return False
def is_use_var(next_child, vars):
    s = ast.unparse(next_child)
    if s != list(vars)[0]:
        g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        for toknum, to_child, _, _, _ in g:
            if to_child.strip() == list(vars)[0]:
                return 1
    return 0
def whether_first_var_is_empty_assign(tree,for_node,vars,const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)


    def whether_contain_var(node,vars):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child == list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time=whether_contain_var(for_node, vars)

    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child,ast.Assign):
            a = []
            append_time += whether_fun_is_append(child, a)
        if isinstance(child, ast.Call):

            def is_call_use_vars(tree, fun_name, vars):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun, "name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) == list(vars)[0]:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, vars)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt
            # break
    remove_ass_flag=1
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if hasattr(node,'body') and isinstance(node.body, list) and node.lineno<for_node.lineno:
            # print("come here node: ",node,ast.unparse(node),isinstance(node,ast.Expr), )
            for child in node.body:
                if isinstance(child, ast.Assign):
                    if ast.unparse(child).strip().split("=")[0].strip() == list(vars)[0] and ast.unparse(child.value) in const_empty_list:
                        assign_stmt = child
                        assign_stmt_lineno = child.lineno
                        flag = 2


                elif assign_stmt_lineno:
                    '''
                    如果ass和for_node在同一层则不需要去除，
                    如果ass的层数在更外层，则我们需要看这一层后面是否使用了vars 如果使用了vars则ass的赋值为空的语句不可以删除
                    如果ass在更外层，在ass所在层的后面节点中，如果存在for节点，并且for节点包含该for_node，则如果for_node后使用了vars，该for_node仍然不可以简化。
                    '''
                    if isinstance(child,ast.For) and child.lineno==for_node.lineno:
                        # print("come here it does not need to remove the ass: \n",ast.unparse(tree))
                        flag=2
                        break
                        #return 2, assign_stmt_lineno, assign_stmt, remove_ass_flag

                    for next_child in ast.walk(child):
                        if not hasattr(next_child, "lineno"):
                            continue
                        if isinstance(next_child,(ast.For,ast.While,ast.AsyncFor)) and next_child.lineno<for_node.lineno and next_child.end_lineno>for_node.end_lineno:
                            # print(">>>>>>>>come here!!!!! ")
                            return 1, assign_stmt_lineno, assign_stmt, remove_ass_flag

                            # for for_child in ast.walk(next_child):
                                # if hasattr(for_child,'lineno'):
                                #     if for_child.lineno>for_node.end_lineno:
                                #         if is_use_var(for_child,vars):
                                #             flag=1
                                #             print(">>>>>>>>come here ")
                                #             return flag, assign_stmt_lineno, assign_stmt, remove_ass_flag
                        elif  next_child.lineno>for_node.end_lineno:
                            if is_use_var(next_child, vars):
                                remove_ass_flag = 0
                                # break

        if assign_stmt_lineno and node!=assign_stmt and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
            # print("come here")

            if is_use_var(node, vars):
                flag = 1
                break
            # if s!=list(vars)[0]:
            #     g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
            #     for toknum, child, _, _, _ in g:
            #         if child.strip() == list(vars)[0]:
            #             flag = 1
            #             # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
            #             break
            #     if flag==1:
            #         break

    return flag,assign_stmt_lineno,assign_stmt,remove_ass_flag
'''
def whether_first_var_is_empty_assign(tree,for_node,vars,const_empty_list=["[]"]):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)


    def whether_contain_var(node,vars):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child == list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time=whether_contain_var(for_node, vars)
    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child,ast.Assign):
            a = []
            append_time += whether_fun_is_append(child, a)
        if isinstance(child, ast.Call):



            def is_call_use_vars(tree, fun_name, vars):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun,"name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) == list(vars)[0]:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, vars)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt
            # break
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if isinstance(node, ast.Assign):
            # print("the assign: ",ast.unparse(node),ast.unparse(node).strip().split("=")[0].strip())
            if ast.unparse(node).strip().split("=")[0].strip()==list(vars)[0] and ast.unparse(node.value) in const_empty_list:
                assign_stmt=node
                assign_stmt_lineno=node.lineno
                flag = 2
        if assign_stmt_lineno and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
            s=ast.unparse(node)

            if s!=list(vars)[0]:
                g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                for toknum, child, _, _, _ in g:
                    if child.strip() == list(vars)[0]:
                        flag = 1
                        # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                        break
                if flag==1:
                    break
    return flag,assign_stmt_lineno,assign_stmt
'''
def else_traverse(if_body,assign_block_list):
    else_body_list = if_body.orelse
    if len(else_body_list) > 1:
        return False
    else:
        else_body_one = else_body_list[0]
        if isinstance(else_body_one, ast.If):
            or_else = else_body_one.orelse
            if not or_else:
                return False
            else:
                return if_else_traverse(else_body_one,assign_block_list)
        return whether_fun_is_append(else_body_one,assign_block_list)



def if_else_traverse(if_body,assign_block_list):
    if_body_list=if_body.body
    # else_body_list=if_body.orelse
    if len(if_body_list)==1:
        if_body_flag =whether_fun_is_append(if_body_list[0],assign_block_list)
    else:
        return False
    #print("if body flag: ",if_body_flag)

    return if_body_flag and else_traverse(if_body,assign_block_list)

def if_traverse(one_body,assign_block_list):
    orelse = one_body.orelse
    if_body=one_body.body
    if not orelse:
        return for_traverse(one_body,assign_block_list)
    else:
        #print("it is if-else node: ", ast.unparse(one_body))
        return if_else_traverse(one_body,assign_block_list)


def for_traverse(node,assign_block_list):
    for_body_list = node.body
    if isinstance(node, ast.For) and node.orelse:
        #print("come here")
        return False
    if len(for_body_list) == 1:
        one_body=for_body_list[0]
        if isinstance(one_body, ast.For):
            return for_traverse(one_body,assign_block_list)
        elif isinstance(one_body, ast.If):
            #print("it is if node")
            return if_traverse(one_body,assign_block_list)

        elif isinstance(one_body, ast.Assign):
            #print("it is one stmt")
            return whether_fun_is_append(one_body,assign_block_list)
        else:

            #print("the for/if sentences has other stmts so cannot transform such as try stmt", ast.unparse(one_body),type(one_body))
            return False
    else:
        return False
'''
beg1 end1
beg2 end2
'''


def get_complicated_for_comprehen_code_list(tree,content):

    #tree = ast.parse(code1)
    code_index_start_end_list = []

    start_lineno=1
    #print("tree: ",tree.body[0].__dict__)
    start_lineno=tree.body[0].lineno
    #print("start_lineno: ",start_lineno)
    #code1 = "\n".join(content.split("\n")[start_lineno-1:])

    for ind_line,node in enumerate(ast.walk(tree)):

        if isinstance(node, ast.For):
            #print("node_line: ",ast.unparse(node),node.lineno, node.end_lineno)
            # if node.lineno!=1674:
            #     continue

            assign_block_list=[]
            #print("it is what: ",node.lineno)
            if for_traverse(node,assign_block_list):
                #print("maybe comrephension")
                vars = set([])
                #print("maybe comrephension: ",node.lineno,ast.unparse(assign_block_list[0][0]))
                for each_block in assign_block_list:
                    var_name=get_var(each_block[0])
                    vars.add(var_name)
                if len(vars)!=1:
                    continue
                # print("vars: ",vars,ind_line,start_lineno)
                # print("node: ", ast.unparse(node))
                flag,assign_stmt_lineno,assign_stmt,remove_ass_flag=whether_first_var_is_empty_assign(tree, node, vars,["dict()","{}"])
                # print("flag: ", flag)
                if flag!=2:
                    print("flag: ")#, ast.unparse(node)
                    continue

                assign_block_list_str = []
                for one_block in assign_block_list:
                    one_block_str = []
                    for e in one_block:
                        one_block_str.append(ast.unparse(e))
                    assign_block_list_str.append(one_block_str)
                code_index_start_end_list.append(
                        [node, assign_stmt,remove_ass_flag])#, node.lineno, node.end_lineno, assign_stmt_lineno, assign_block_list_str])

                # print(">>>>>>>>>>>>>>>>>>>>>>it is comprehension code1\n", remove_ass_flag,node.lineno, node.end_lineno,assign_stmt_lineno,ast.unparse(node))
                # code_index_start_end_list.append([ast.unparse(node), node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
    #print("len: ", len(code_index_start_end_list))
    code_index_start_end_list = extract_compli_for_comprehension_logic.filter_overlap(code_index_start_end_list)
    # code_index_start_end_list = filter_overlap(code_index_start_end_list)
    # print("len: ",len(code_index_start_end_list))
    return code_index_start_end_list
def save_one_repo(repo_name):
    count=0
    one_repo_chained_comparison_code_list = []
    dict_file = dict()
    for file_info in dict_repo_file_python[repo_name]:
        file_path = file_info["file_path"]
        file_html = file_info["file_html"]
        # if file_html != "https://github.com/networkx/networkx/tree/master/networkx/algorithms/connectivity/kcomponents.py":#"https://github.com/IDSIA/sacred/tree/master/sacred/observers/mongo.py":  # "https://github.com/cloud-custodian/cloud-custodian/tree/master/c7n/provider.py":#"https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":#"https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":##"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/nccgroup/ScoutSuite/tree/master/ScoutSuite/providers/aws/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/localstack/localstack/tree/master/localstack/services/awslambda/lambda_api.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/aws/aws-cli/tree/master/awscli/compat.py":#"https://github.com/spulec/moto/tree/master/moto/config/models.py":#"https://github.com/cookiecutter/cookiecutter/tree/master/cookiecutter/hooks.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/greedy_algorithm/fractional_knapsack.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/strings/anagram.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/cloudtools/troposphere/tree/master/troposphere/validators.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":  # "https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
        #     continue
        # if file_html != "https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":  #"https://github.com/cloud-custodian/cloud-custodian/tree/master/c7n/provider.py":  # "https://github.com/networkx/networkx/tree/master/networkx/generators/joint_degree_seq.py":#"https://github.com/bndr/pipreqs/tree/master/pipreqs/pipreqs.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":##"https://github.com/microsoft/nni/tree/master/nni/tools/nnictl/nnictl_utils.py":#"https://github.com/networkx/networkx/tree/master/networkx/readwrite/json_graph/adjacency.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/aws/aws-cli/tree/master/awscli/customizations/s3/subcommands.py":#"https://github.com/nccgroup/ScoutSuite/tree/master/ScoutSuite/providers/aws/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/localstack/localstack/tree/master/localstack/services/awslambda/lambda_api.py":#"https://github.com/localstack/localstack/tree/master/localstack/utils/common.py":#"https://github.com/aws/aws-cli/tree/master/awscli/compat.py":#"https://github.com/spulec/moto/tree/master/moto/config/models.py":#"https://github.com/cookiecutter/cookiecutter/tree/master/cookiecutter/hooks.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/greedy_algorithm/fractional_knapsack.py":#"https://github.com/OmkarPathak/pygorithm/tree/master/pygorithm/strings/anagram.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/cloudtools/troposphere/tree/master/troposphere/validators.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/ytdl-org/youtube-dl/tree/master/youtube_dl/utils.py":#"https://github.com/sympy/sympy/tree/master/sympy/crypto/crypto.py":#"https://github.com/yt-dlp/yt-dlp/tree/master/yt_dlp/utils.py":#"https://github.com/amperser/proselint/tree/master/proselint/tools.py":#"https://github.com/pytransitions/transitions/tree/master/transitions/extensions/nesting.py":#"https://github.com/microsoft/nni/tree/master/nni/algorithms/hpo/networkmorphism_tuner/graph_transformer.py":  # "https://github.com/pymc-devs/pymc/tree/master/pymc/sampling.py":#"https://github.com/smicallef/spiderfoot/tree/master//sflib.py":#"https://github.com/smicallef/spiderfoot/tree/master//sfwebui.py":#
        #     continue
        # if file_html!="https://github.com/python-poetry/poetry/tree/master/poetry/puzzle/solver.py":#"https://github.com/Rockyzsu/stock/tree/master//fupan.py":#"https://github.com/OctoPrint/OctoPrint/tree/master/src/octoprint/server/views.py":
        #     continue
        print("file_html: ", file_html)
        try:
            content = util.load_file_path(file_path)
        except:
            print(f"{file_path} is not existed!")
            continue

        try:
            file_tree = ast.parse(content)
            ana_py = ast_util.Fun_Analyzer()
            ana_py.visit(file_tree)
            # one_file_chained_comparison_code_list = []
            #print("func number: ",file_html,len(ana_py.func_def_list))
            dict_class = dict()
            for tree, class_name in ana_py.func_def_list:
                # if 1:
                #print("come here",ast.unparse(tree))
                new_code_list = get_complicated_for_comprehen_code_list(tree,content)
                for ind, (for_node, assign_node,remove_ass_flag) in enumerate(new_code_list):
                    new_code_list[ind].append(
                        transform_comprehension_dict_compli_to_simple.transform(for_node, assign_node))
                # for old_code,old_ass,new_code in new_code_list:
                #     print("old_code,old_ass,new_code: ",ast.unparse(old_code),old_ass,ast.unparse(new_code))

                # one_file_chained_comparison_code_list.extend(get_complicated_for_comprehen_code_list(tree,content))
                ast_util.set_dict_class_code_list(tree, dict_class, class_name, new_code_list)

                # count+=len(one_file_chained_comparison_code_list)
            dict_file[file_html] = dict_class
            #print(dict_class)
            # if one_file_chained_comparison_code_list:
            #     one_repo_chained_comparison_code_list.append(
            #         [one_file_chained_comparison_code_list, file_path, file_html])
                # print("one_file_truth_value_test_code_list: ",one_repo_chained_comparison_code_list)
                    # break
        except SyntaxError:
            print("the file has syntax error")
            continue
        except ValueError:
            print("the file has value error: ", file_html)#content,
            continue
        except:
            traceback.print_exc()
        # break
    #print("count: ",count)
    util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
    print("save successfully! ", count,save_complicated_code_dir_pkl + repo_name)


if __name__ == '__main__':
    code = '''
# def func_1():
#     components = {}
#     for i in range(2):
#         print(1)
#     components['trend'] = get_forecast_component_plotly_props(
#         m, fcst, 'trend', uncertainty, plot_cap)
#     if m.train_holiday_names is not None and 'holidays' in fcst:
#         components['holidays'] = get_forecast_component_plotly_props(
#             m, fcst, 'holidays', uncertainty)
#     
#     regressors = {'additive': False, 'multiplicative': False}
#     for name, props in m.extra_regressors.items():
#         regressors[props['mode']] = True
#     for mode in ['additive', 'multiplicative']:
#         if regressors[mode] and 'extra_regressors_{}'.format(mode) in fcst:
#             components['extra_regressors_{}'.format(mode)] = get_forecast_component_plotly_props(
#                 m, fcst, 'extra_regressors_{}'.format(mode))
payload={}
for field in model.__searchable__:
    payload[field] = getattr(model, field)
'''
    # dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")
    #
    # save_one_repo("Real-Time-Voice-Cloning", code1)

    '''
    # print("code1: ",code1)
    tree = ast.parse(code1)
    #code1 = ast.unparse(tree)
    #print("code1: ", code1)

    code_list=get_complicated_for_comprehen_code_list(tree,code1)
    print("len: ",len(code_list))
    for e in code_list:
        print(">>>>>code1： ", e[0])
        print("block: ", e[-1])
        for each in  e[-1]:
            print(each)

        print("----------END------------")
    '''
    # test_code="a=dict()"
    # test_tree = ast.parse(test_code)
    # print(ast.unparse(test_tree))

    # layer_node_list = []
    # ast_util.extract_ast_cur_layer_node(tree, layer_node_list)
    '''
    get comprehension node
    Block=> Assign;...; For
    For=>”for” ( For| If|If-else|body)
    If=>”if” (body| For| If| If-else)
    If-else=>”if” body “else” (body| If-else)
    body=> (Assign|Call)*;Append*
    '''
    # 对于
    save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_dict/"
    # save_complicated_code_dir = util.data_root + "complicated_code_dir/for_comrephension_dict_complicated_only_one_stmt/"
    dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")

    #'''
    # dict_repo_file_python = util.load_json(util.data_root, "python3_1000repos_files_info")
    repo_list = []
    for ind, repo_name in enumerate(dict_repo_file_python):
        # if repo_name!="networkx":#"sacred":#"networkx":#"cloud-custodian":#"poetry":#"OctoPrint":#"keras-bert":#"Legofy":
        #     continue
        # print("repo_name: ", repo_name)
        repo_list.append(repo_name)
    # save_one_repo(repo_list[0])

    #'''
    pool = newPool(nodes=30)
    pool.map(save_one_repo, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    #'''

    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            # print("code1",code1)
                            result_compli_for_else_list.append(
                                [repo_name, file_html, cl, me, ast.unparse(code[0]), ast.unparse(code[1])])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "complicated_code_dir_pkl/for_compre_dict.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html", "class_name", "me_name", "for_code", "assign_code"])














import ast
import tokenize
import traceback

import os

p_name="python-ast-explorer-master"
abs_path=os.path.abspath(os.path.dirname(__file__))
abs_path=abs_path[:abs_path.index("extract_transform_complicate_code_new")]
# print()
import sys
sys.path.append(abs_path)
sys.path.append(abs_path+"extract_transform_complicate_code_new/")
sys.path.append(abs_path+"extract_transform_complicate_code_new/comprehension/")
from RefactoringIdioms.extract_simp_cmpl_data import ast_util
# from code1.extract_simp_cmpl_data import ast_util
import extract_compli_for_comprehension_logic
from RefactoringIdioms.transform_c_s import transform_comprehension_compli_to_simple
from tokenize import tokenize
from io import BytesIO
# from pathos.multiprocessing import ProcessingPool as newPool
'''
get comprehension node
Block=> Assign;...; For
For=>”for” ( For| If|If-else|body)
If=>”if” (body| For| If| If-else)
If-else=>”if” body “else” (body| If-else)
body=> (Assign|Call)*;Append*
并且里面只有一个写变量
for

    b.append()
    
    a.append(b)

'''
const_empty=" = []"
def find_complicate_list_comprehension(code):

    pass
def whether_block_call_assign(node_list,assign_block_list):
    contain_append_flag=0
    #print(ast.unparse(node_list))
    for e_body in node_list:
        #print("code1: ",ast.unparse(e_body),e_body.__dict__)

        if (not (isinstance(e_body, ast.Expr) and isinstance(e_body.value, ast.Call))) and (not isinstance(e_body, ast.Assign)):
            #print("the for sentences has other stmts so cannot transform such as try stmt: ", type(e_body),ast.unparse(e_body))

            return False
        if isinstance(e_body, ast.Expr) and isinstance(e_body.value, ast.Call):
            call_name = ""
            call_front=e_body.value.func
            # print(call_front.__dict__,ast.unparse(call_front))
            if isinstance(call_front, ast.Name):
                call_name = ast.unparse(call_front)
            elif isinstance(call_front, ast.Attribute):
                call_name = call_front.attr
            else:
                call_name = ast.unparse(call_front)
            if call_name=="append":
                contain_append_flag=1

    if contain_append_flag:
        assign_block_list.append(node_list)
        return True
    return False
def get_func_name(one_body):
    pre_name,call_name=None,None
    if isinstance(one_body, ast.Expr) and isinstance(one_body.value, ast.Call):
        # print("come here")
        call_front = one_body.value.func
        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
            pre_name=ast.unparse(call_front.value)
        else:
            call_name = ast.unparse(call_front)
    return pre_name,call_name


def whether_fun_is_append(one_body,assign_block_list):
    #print("one_body: ",ast.unparse(one_body),one_body.__dict__)
    if isinstance(one_body, ast.Expr) and isinstance(one_body.value, ast.Call) :
        # print("come here")
        call_front = one_body.value.func

        # print("call dict: ", call_front.__dict__)
        call_name = ""
        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
        else:
            call_name = ast.unparse(call_front)
        #print("the func name: ", call_name)
        if call_name == "append":
            assign_block_list.append([one_body])
            return True
    elif isinstance(one_body,ast.Call):
        call_front = one_body.func

        # print("call dict: ", call_front.__dict__)
        call_name = ""
        if isinstance(call_front, ast.Name):
            call_name = ast.unparse(call_front)
        elif isinstance(call_front, ast.Attribute):
            call_name = call_front.attr
        else:
            call_name = ast.unparse(call_front)
        # print("the func name: ", call_name)
        if call_name == "append":
            assign_block_list.append([one_body])
            return True

    return False

def else_traverse(if_body,assign_block_list):
    else_body_list = if_body.orelse
    if len(else_body_list) > 1:
        return False
    else:
        else_body_one = else_body_list[0]
        if isinstance(else_body_one, ast.If):
            or_else = else_body_one.orelse
            if not or_else:
                return False
            else:
                return if_else_traverse(else_body_one,assign_block_list)
        return whether_fun_is_append(else_body_one,assign_block_list)



def if_else_traverse(if_body,assign_block_list):
    if_body_list=if_body.body
    # else_body_list=if_body.orelse
    if len(if_body_list)==1:
        if_body_flag =whether_fun_is_append(if_body_list[0],assign_block_list)
    else:
        return False
    #print("if body flag: ",if_body_flag)

    return if_body_flag and else_traverse(if_body,assign_block_list)
def if_traverse(one_body,assign_block_list):
    orelse = one_body.orelse
    if_body=one_body.body
    if not orelse:
        return for_traverse(one_body,assign_block_list)
    else:
        #print("it is if-else node: ", ast.unparse(one_body))
        return if_else_traverse(one_body,assign_block_list)



def for_traverse(node,assign_block_list):
    for_body_list = node.body
    if isinstance(node, ast.For) and node.orelse:
       # print("come here")
        return False
    if len(for_body_list) == 1:
        one_body=for_body_list[0]
        if isinstance(one_body, ast.For):
            #print("come here for_traverse")
            return for_traverse(one_body,assign_block_list)
        elif isinstance(one_body, ast.If):
            #print("it is if node")
            return if_traverse(one_body,assign_block_list)

        elif isinstance(one_body, ast.Expr):
            #print("come here for_traverse and it is an Expr",whether_fun_is_append(one_body,assign_block_list))
            return whether_fun_is_append(one_body,assign_block_list)
        else:

            #print("the for/if sentences has other stmts so cannot transform such as try stmt", ast.unparse(one_body),type(one_body))
            return False
    else:
        return False
'''
beg1 end1
beg2 end2
'''
def filter_overlap(code_index_list):
    no_overlap_list=[]
    if len(code_index_list)==1:
        return code_index_list
    for i in range(len(code_index_list)):
        code=code_index_list[i]
        beg=code[0].lineno
        end=code[0].end_lineno
        for j in range(len(code_index_list)):
            if i==j:
                continue
            fuzhu_beg = code_index_list[j][0].lineno
            fuzhu_end = code_index_list[j][0].end_lineno
            if beg>=fuzhu_beg and end<=fuzhu_end:
                break
        else:
            no_overlap_list.append(code)

    return no_overlap_list
'''
1. 当前for_node中处理对vars进行append操作外，不应该使用vars
    如果节点中存在Call节点还需要判断Call中是否使用了vars
2. 判断tree中在for_node之前的节点是否有vars的赋值语句，如果存在这样的赋值语句，还需要判断在这个赋值语句父母中for_node节点后面是否还使用了vars
    如果使用了vars则这个赋值节点不应该被删除

    
'''
def whether_first_var_is_empty_assign(tree,for_node,vars):
    assign_stmt=None
    assign_stmt_lineno=None
    flag=0
    # start_lineno=0
    # print("the var: ",vars)


    def whether_contain_var(node,vars):
        count=0
        s = ast.unparse(node)
        if s != list(vars)[0]:
            g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string

            for toknum, child, _, _, _ in g:
                if child == list(vars)[0]:
                    count+=1
                    # print("yes use more than one time: ", s)
        return count

    time=whether_contain_var(for_node, vars)
    append_time = 0
    for child in ast.walk(for_node):

        # if ast.unparse(child)==list(vars)[0]:
        #     time+=1
        #     if time-append_time > 0:
        #         print("come here the for node use var more than one time ",list(vars)[0])
        #         flag = 1
        #         break
        if isinstance(child, ast.Call):
            a = []
            append_time += whether_fun_is_append(child, a)

            def is_call_use_vars(tree, fun_name, vars):
                # print(">>>>>>.come is_call_use_vars: ", fun_name)
                ana_py_fun = ast_util.Fun_Analyzer()
                ana_py_fun.visit(tree)
                for tree_fun, class_name in ana_py_fun.func_def_list:
                    if hasattr(tree_fun, "name"):
                        each_fun_name = tree_fun.name
                        # print(">>>>>>each_fun_name: ", fun_name)
                        if each_fun_name == fun_name:
                            for child in ast.walk(tree_fun):
                                if ast.unparse(child) == list(vars)[0]:
                                    # print("yes the call use vars: ", each_fun_name)
                                    return 1
                return 0

            fun_name = ast.unparse(child.func).split(".")[-1]
            call_use_var_flag = is_call_use_vars(tree, fun_name, vars)
            if call_use_var_flag:
                flag = 1
                return flag, assign_stmt_lineno, assign_stmt
        # time += whether_contain_var(child, vars)
    if time - append_time > 0:
            # print("come here the for node use var more than one time ", list(vars)[0])
            flag = 1
            return flag, assign_stmt_lineno, assign_stmt
            # break
    remove_ass_flag=1
    for node in ast.walk(tree):
        if not hasattr(node,"lineno"):
            continue

        if node.lineno==for_node.lineno:
            break
        if hasattr(node,'body') and isinstance(node.body, list) and node.lineno<for_node.lineno:
            # print("come here node: ",node,ast.unparse(node),isinstance(node,ast.Expr), )
            for child in node.body:
                if isinstance(child, ast.Assign):
                    if ast.unparse(child).strip().split("=")[0].strip() == list(vars)[0] and ast.unparse(child.value) == "[]":
                        assign_stmt = child
                        assign_stmt_lineno = child.lineno
                        flag = 2


                elif assign_stmt_lineno:
                    for next_child in ast.walk(child):
                        if not hasattr(next_child,"lineno"):
                            continue
                        if next_child.lineno>for_node.end_lineno:
                            s = ast.unparse(next_child)
                            if s != list(vars)[0]:
                                g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                                for toknum, to_child, _, _, _ in g:
                                    if to_child.strip() == list(vars)[0]:
                                        remove_ass_flag =0
                                        # print("yes in the assign block it use more than one time: ", vars, s, "\n>>>>>>>>>", ast.unparse(for_node))
                                        break
                                else:
                                    continue
                                break

        if assign_stmt_lineno and node!=assign_stmt and node.end_lineno<for_node.lineno and node.lineno>assign_stmt_lineno :
            s=ast.unparse(node)

            if s!=list(vars)[0]:
                g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
                for toknum, child, _, _, _ in g:
                    if child.strip() == list(vars)[0]:
                        flag = 1
                        # print("yes use more than one time: ",vars,s,"\n>>>>>>>>>",ast.unparse(for_node))
                        break
                if flag==1:
                    break
        # 判断跟compreh不属于同一个父母的在for_node后面的代码片段是否使用了变量vars
        # if assign_stmt_lineno and node!=assign_stmt and node.lineno>ass_parent.end_lineno:
        #     s = ast.unparse(node)
        #
        #     if s != list(vars)[0]:
        #         g = tokenize(BytesIO(s.encode('utf-8')).readline)  # tokenize the string
        #         for toknum, child, _, _, _ in g:
        #             if child.strip() == list(vars)[0]:
        #                 remove_ass_flag = 0
        #                 print("no we cannot remove the ass: ","\n>>>>>>>>>",ast.unparse(for_node),"\n>>>>>>>>>",ast.unparse(node))
        #                 break
        #         if remove_ass_flag == 0:
        #             break

    return flag,assign_stmt_lineno,assign_stmt,remove_ass_flag

def get_complicated_for_comprehen_code_list(tree,content):
    #code1 = ast.unparse(tree)
    # print("come get java-style python code",ast.unparse(tree))
    code_index_start_end_list = []

    start_lineno=tree.body[0].lineno
    #code1 = "\n".join(content.split("\n")[start_lineno - 1:])
    for ind_line,node in enumerate(ast.walk(tree)):
        # print("come walk the tree",ast.unparse(node))
            #print("start_line: ",node.lineno)
        if isinstance(node, ast.For):
            # print("node_line: ",ast.unparse(node),node.lineno, node.end_lineno)
            # if node.lineno!=1674:
            #     continue

            assign_block_list=[]
            #print("it is what: ",for_traverse(node,assign_block_list))
            if for_traverse(node,assign_block_list):#判断for语句是否满足For_Match
                # print("maybe comrephension: ",ast.unparse(node))
                '''
                判断是否是同一个变量的append
                '''
                vars = set([])
                for each_block in assign_block_list:
                    pre_name,call_name=get_func_name(each_block[0])
                    vars.add(pre_name)
                if len(vars)!=1:
                    continue
                # print("for node: ",ast.unparse(node),node.lineno)
                '''
                判断是否是同一个变量的append
                '''
                flag,assign_stmt_lineno,assign_stmt,remove_ass_flag= extract_compli_for_comprehension_logic.whether_first_var_is_empty_assign(tree, node, vars)
                if flag!=2:
                    print("it does not have the var or the var has been used more than one time",remove_ass_flag)
                    continue
                #print("flag: ", assign_stmt,assign_stmt_lineno,suite)
                assign_block_list_str = []
                for one_block in assign_block_list:
                    one_block_str = []
                    for e in one_block:
                        one_block_str.append(ast.unparse(e))
                    assign_block_list_str.append(one_block_str)
                # print(">>>>>>>>>>>>>>>>>>>>>>it is complicated comprehension code1\n", remove_ass_flag,ast.unparse(assign_stmt),"\n",ast.unparse(node),node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str,assign_stmt)
                # print(assign_stmt+"\n"+ast.unparse(node))
                code_index_start_end_list.append([node,assign_stmt,remove_ass_flag])
                # code_index_start_end_list.append([ast.unparse(node),node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str, assign_stmt])
    #print("len: ", len(code_index_start_end_list))
    code_index_start_end_list = filter_overlap(code_index_start_end_list)
    # print("len: ",len(code_index_start_end_list))

    return code_index_start_end_list

def get_list_compreh(content):
    code_pair_list=[]

    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)
        # print("come here",ana_py.func_def_list)
        for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name=tree.name
            except:
                me_name=""
            new_code_list = get_complicated_for_comprehen_code_list(tree, content)
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code= transform_comprehension_compli_to_simple.transform(for_node, assign_node)
                complete_new_code=ast.unparse(new_code) if remove_ass_flag else ast.unparse(assign_node) + "\n"+ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                # print(">>>>>assign_node_content:\n",[assign_node_content, for_node_content])
                if remove_ass_flag:
                    line_list = []

                    line_list.append([(assign_node.lineno,assign_node.col_offset), (assign_node.end_lineno,assign_node.end_col_offset)])
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    code_pair_list.append([class_name, me_name, [assign_node_content, for_node_content],
                                           ['', ast.unparse(new_code)],
                                           line_list])
                    # code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                    #                        ['', ast.unparse(new_code)],
                    #                        line_list])
                else:
                    line_list = []
                    line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                    # code_pair_list.append(
                    #     [class_name, me_name, [ast.unparse(for_node)], [ast.unparse(new_code)], line_list])
                    code_pair_list.append(
                        [class_name, me_name, [for_node_content], [ast.unparse(new_code)], line_list])

                # if remove_ass_flag:
                #     line_list = []
                #     line_list.append([assign_node.lineno, assign_node.end_lineno])
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                #                            ['', ast.unparse(new_code)],
                #                            line_list])
                # else:
                #     line_list = []
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name,me_name,[ast.unparse(for_node)], [ast.unparse(new_code)],line_list])

        return code_pair_list

    except:
        traceback.print_exc()
        code_pair_list=["syntax error in code"]
        return code_pair_list

def get_list_compreh_part_fragment(content):
    code_pair_list=[]

    # print(content)
    try:

            tree = ast.parse(content)

        # print("come here",ana_py.func_def_list)
        # for tree, class_name in ana_py.func_def_list:
            # print("come method",ast.unparse(tree))
            try:
                me_name=tree.name
            except:
                me_name=""
            new_code_list = get_complicated_for_comprehen_code_list(tree, content)
            # print("come method", new_code_list,ast.unparse(tree))
            for ind, (for_node, assign_node, remove_ass_flag) in enumerate(new_code_list):
                new_code= transform_comprehension_compli_to_simple.transform(for_node, assign_node)
                complete_new_code=ast.unparse(new_code) if remove_ass_flag else ast.unparse(assign_node) + "\n"+ast.unparse(new_code)
                assign_node_content = ast.get_source_segment(content, assign_node)
                for_node_content = ast.get_source_segment(content, for_node)
                # print(">>>>>for_node_content:\n",for_node_content)
                if 1:
                # if target_node_str[-1]==ast.unparse(assign_node)+"\n"+ast.unparse(for_node):
                    if remove_ass_flag:
                        line_list = []

                        line_list.append([(assign_node.lineno,assign_node.col_offset), (assign_node.end_lineno,assign_node.end_col_offset)])
                        line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                        code_pair_list.append(['', me_name, [assign_node_content, for_node_content],
                                               ['', ast.unparse(new_code)],
                                               line_list])
                        # code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                        #                        ['', ast.unparse(new_code)],
                        #                        line_list])
                    else:
                        line_list = []
                        line_list.append([(for_node.lineno,for_node.col_offset), (for_node.end_lineno,for_node.end_col_offset)])
                        # code_pair_list.append(
                        #     [class_name, me_name, [ast.unparse(for_node)], [ast.unparse(new_code)], line_list])
                        code_pair_list.append(
                            ['', me_name, [for_node_content], [ast.unparse(new_code)], line_list])

                # if remove_ass_flag:
                #     line_list = []
                #     line_list.append([assign_node.lineno, assign_node.end_lineno])
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name, me_name, [ast.unparse(assign_node), ast.unparse(for_node)],
                #                            ['', ast.unparse(new_code)],
                #                            line_list])
                # else:
                #     line_list = []
                #     line_list.append([for_node.lineno, for_node.end_lineno])
                #     code_pair_list.append([class_name,me_name,[ast.unparse(for_node)], [ast.unparse(new_code)],line_list])

            return code_pair_list

    except:
        # traceback.print_exc()
        # code_pair_list=["syntax error in code"]
        return code_pair_list


if __name__ == '__main__':
    code = '''
# a=[]
# for i in range(4):
#     a.append(i)
# a=[]
for i in range(3):
    call(a)
a=[]
b=[]
for i in range(4):
    if i>3:
        for j in range(a):
            # i=i**2
            # a.append(j)
            b.append(i)
#     # elif i>4:
#     #     a.append(i)
# for i in range(4):
#      if i>3:
#          if i>3:
#              a.append(i)
#          else:
#               a.append(i)

for i in range(4):
     # if i>3:
         if i>3:
             a.append(i)
         else:
              a.append(i)
# a=[]
for i in range(3):
    b=[]
    for i in range(3): # 这个节点里包含 除了assign和Call的节点 因为在comprehension的繁杂代码中不可以再有for语句
         if i>3:
             b.append(i)
    a.append(b)  
threads = []
for _ in range(4):
    threads.append(threading.Thread(target=test)) 
'''

    '''

    tree = ast.parse(code1)
    code_list=get_complicated_for_comprehen_code_list(tree, code1)
    #code_list=get_complicated_for_comprehen_code_list(code1)
    print("len: ",len(code_list))
    for e in code_list:
        print(">>>>>code1： ", e[0])
        print("block: ", e[-1])
        # for each in  e[-1]:
        #     print(each)

        print("----------END------------")
    # test_code="a=[]"
    # test_tree = ast.parse(test_code)
    # print(ast.unparse(test_tree))
    '''
    # layer_node_list = []
    # ast_util.extract_ast_cur_layer_node(tree, layer_node_list)
    '''
    get comprehension node
    Block=> Assign;...; For
    For=>”for” ( For| If|If-else|body)
    If=>”if” (body| For| If| If-else)
    If-else=>”if” body “else” (body| If-else)
    body=> (Assign|Call)*;Append*
    '''
    # 对于

    # save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl/for_compre_list/"

    # save_complicated_code_dir = util.data_root + "complicated_code_dir/for_comrephension_list_complicated_only_one_stmt/"
    #'''
    #dict_repo_file_python = util.load_json(util.data_root, "python3_repos_files_info")
    # dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")

    code='''
def f():
        a = []
        for e in range(10):
            a.append(e)
'''
    print(get_list_compreh(code))
    # repo_name_list=[]
    # for repo_name in dict_repo_file_python:
    #
    #     repo_name_list.append(repo_name)
    #     if repo_name=="hosts":
    #         print("Come here repo")
    #         save_one_repo(repo_name)

    # print("repo num: ", len(list(dict_repo_file_python.keys())))
    # count_complicated_code = 0
    # count_repo = 0
    # repo_list = []
    # for ind, repo_name in enumerate(dict_repo_file_python):
    #     # if repo_name!="pipreqs":#"proselint":#"networkx":#"nni":#"aws-cli":#"youtube-dl":#"localstack":#"transitions":#"youtube-dl":#"hypothesis":
    #     #     continue
    #     # if repo_name!="youtube-dl":#"transitions":#"metrics":##"youtube-dl":#"proselint":#"localstack":#"shuup":#"Real-Time-Voice-Cloning":#"keras-bert":#"Legofy":
    #     #     continue
    #     #print("repo_name: ", repo_name)
    #     repo_list.append(repo_name)
    # print("repo num: ", len(repo_list))
    import time
    start_time = time.time()
    # save_one_repo(repo_list[0])
    '''
    pool = newPool(nodes=30)
    pool.map(save_one_repo, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
    pool.close()
    pool.join()
    '''
    '''
    files_num_list = []
    star_num_list = []
    contributor_num_list = []
    count_repo, file_count, me_count, code_count = 0, 0, 0, 0
    file_list = set([])
    repo_code_num = dict()
    result_compli_for_else_list = []
    all_count_repo, all_file_count, all_me_count = 0, 0, 0
    for file_name in os.listdir(save_complicated_code_dir_pkl):
        all_count_repo += 1
        repo_name = file_name[:-4]
        # files_num_list.append(repo_files_info[repo_name])
        # star_num_list.append(repo_star_info[repo_name])
        # contributor_num_list.append(repo_contributor_info[repo_name])

        complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)

        repo_file_count, repo_me_count, repo_code_count, repo_all_file_count, repo_all_me_count = complicated_code_util.get_code_count(
            complicate_code)
        # for code_list, file_path, file_html in complicate_code:
        code_count += repo_code_count
        file_count += repo_file_count
        me_count += repo_me_count
        all_file_count += repo_all_file_count
        all_me_count += repo_all_me_count
        repo_exist = 0
        for file_html in complicate_code:
            for cl in complicate_code[file_html]:
                for me in complicate_code[file_html][cl]:
                    if complicate_code[file_html][cl][me]:
                        repo_exist = 1
                        for code in complicate_code[file_html][cl][me]:
                            # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
                            #                code_index_start_end_list.append([node,assign_stmt,node.lineno, node.end_lineno,assign_stmt_lineno,assign_block_list_str])
                            # result_compli_for_else_list.append(
                            #     [repo_name, file_html, cl, me, code1[1].lineno,
                            #      ast.unparse(code1[1]) +"\n"+ ast.unparse(code1[0]), ast.unparse(code1[-1]), code1[2]])
                            result_compli_for_else_list.append(
                                [repo_name, file_html, cl, me, code[1].lineno,
                                 ast.unparse(code[1]) + "\n" + ast.unparse(code[0]), ast.unparse(code[-1]), code[2]])

            # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
        count_repo += repo_exist

    # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
    # print(a)
    # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
    # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
    # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
    # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
    print("count: ", count_repo, code_count, file_count, me_count, all_count_repo, all_file_count, all_me_count)
    import random

    random.shuffle(result_compli_for_else_list)
    util.save_csv(util.data_root + "result_csv/for_comprehen_list.csv",
                  result_compli_for_else_list[:400],
                  ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code",
                   "remove_ass_flag"])

    end_time = time.time()
    print("total time: ", end_time - start_time)
    '''
    # 1 156 2943 100 1 2990 40102 salt
    # 791 2010 103165 1291 800 121348 1192868
    # util.save_csv(util.data_root + "complicated_code_dir_pkl/for_else.csv", result_compli_for_else_list,
    #               ["repo_name", "file_html","class_name", "me_name", "for_code", "assign_code"])

    '''
    count = 0
    result_compli_for_else_list = []
    for file_name in os.listdir(save_complicated_code_dir):
        complicate_code = util.load_json(save_complicated_code_dir, file_name[:-5])
        for each_file in complicate_code:

            # for code_list, file_path,file_html in each_file:
            #
            #     print("count: ",code_list)
            code_list = each_file[0]
            file_path = each_file[1]
            file_html = each_file[2]
            count += len(code_list)
            # print("count: ", count)
            for code1 in code_list:
                #print("code1: ",code1)
                repo_name = file_html.split("/")[4]
                #print("code1: ",code1)
                result_compli_for_else_list.append(
                    [repo_name, code1[-2]+"\n"+ast.unparse()code1[0]+"\n"+code1[-1], str(code1[1])+" "+str(code1[2])+" "+str(code1[3]), file_html, file_path])

        #     print("one code1: ",repo_name,code1,file_html,file_path)
        #     break
        # break
        # print("file: ",file_name)
        # break
    print("count: ", count, len(os.listdir(save_complicated_code_dir)))#,result_compli_for_else_list)
    # util.save_csv(util.data_root + "complicated_code_dir/for_comrephension_list_complicated_only_one_stmt.csv",
    #               result_compli_for_else_list, ["repo_name", "code1", "start_end_assign_ind", "file_html", "file_path"])
    '''














import sys,ast,os

abs_path=os.path.abspath(os.path.dirname(__file__))
# print("abs_path: ",abs_path)
pack_path="/".join(abs_path.split("/")[:-1])
# print(pack_path)
sys.path.append(pack_path)
# sys.path.append("/mnt/zejun/smp/code1/")
import util
from sympy import *

def build_star_node(arg_value_list,step):
    # star_node = ast.Starred()
    # star_node.value = ast.Subscript()
    # print("star node: ", ast.unparse(star_node))
    # star_node.value.value = arg_value_list[0].value
    star_node=None

    # star_node.value.slice = ast.Slice()
    # star_node.value.slice.lower = arg_value_list[0].slice
    # star_node.value.slice.upper = arg_value_list[-1].slice
    # star_node.value.slice.step = step
    # print("star node: ",ast.unparse(star_node))
    end=ast.unparse(arg_value_list[-1].slice)
    # if end.isdigit():
    #     end=str(int(end)+1)
    # else:
    end = str(sympify(end + '+'+str(step)))
        # end=end+"+1"
    start_str=ast.unparse(arg_value_list[0].slice)
    str_step=str(step)
    if start_str!='0' and str_step!='1':
        a=["*",ast.unparse(arg_value_list[0].value),"[",ast.unparse(arg_value_list[0].slice),":",end,":",str(step),"]"]
    elif start_str=='0' and str_step!='1':
        a=["*",ast.unparse(arg_value_list[0].value),"[",":",end,":",str(step),"]"]
    elif start_str != '0' and str_step=='1':
        a=["*",ast.unparse(arg_value_list[0].value),"[",ast.unparse(arg_value_list[0].slice),":",end,"]"]
    elif start_str == '0' and str_step=='1':
        a=["*",ast.unparse(arg_value_list[0].value),"[",":",end,"]"]

    star_node=ast.parse("".join(a)).body[0]
    # print("star node: ", star_node,ast.unparse(star_node))
    return star_node
def transform_var_unpack_call_each_args(arg_info_list):

    # all_ind_list=item[0]
    # step=item[2]
        '''
           remove corresponding elements in the args of ind_list
           and insert  replaced them with a starred node
        '''
        bia = 0
    # for ind,arg_info_list in enumerate(item):
        step = arg_info_list[3]
        ind_list=arg_info_list[1]
        # node=arg_info_list[-2]
        node=arg_info_list[2]
        # print("node: ",node,ind_list,step)
        args = node.args

        beg=ind_list[0]+bia
        arg_value_list=[]
        for i in range(len(ind_list)):
            arg_value_list.append(args.pop(beg))


        # print("args: ", arg_value_list)
        star_node= build_star_node(arg_value_list,step)


        # print("star_node: ", ast.unparse(star_node))
        return star_node
    #     args.insert(beg,star_node)
    #     bia+=1-len(ind_list)
    #
    # print("code1: ",ast.unparse(node))


def transform_var_unpack_call(node,item):

    # all_ind_list=item[0]
    # step=item[2]
    '''
       remove corresponding elements in the args of ind_list
       and insert  replaced them with a starred node
    '''
    bia = 0
    for ind,arg_info_list in enumerate(item):
        step = arg_info_list[3]
        ind_list=arg_info_list[1]
        # node=arg_info_list[2]
        # print("node: ",node,ind_list,step)
        args = node.args
        # print("args: ", args)
        beg=ind_list[0]+bia
        arg_value_list=[]
        for i in range(len(ind_list)):
            arg_value_list.append(args.pop(beg))
        star_node= build_star_node(arg_value_list,step)

        args.insert(beg,star_node)
        bia+=1-len(ind_list)

    # print("code1: ",ast.unparse(node))

    pass


if __name__ == '__main__':
    code='''
upfirdn2d_native(input, kernel, up, up, down, down, pad[0], pad[1], pad[0], pad[1])
ConvBNLayer(out_channels[1], out_channels[1], 3, 1, act='relu', name='fpn_up_g1_1')
sort_by_key_ir(ins[0], ins[1], outs[0], outs[1], outs[2], outs[3], axis, is_ascend)
a=axes.text( rect.xy[1],rect.xy[0], rect.xy[1], labels[I:],labels[i+1],labels[2],bbox=dict(facecolor=color, lw=0))
# a=axes.text( rect.xy[0],rect.xy[1], rect.xy[1], labels[I:],labels[i+1],labels[2],bbox=dict(facecolor=color, lw=0))
# b=func(a[i],a[i+1],c[1])
# b=func(a[i+1],a[i],a[i+1],c[1])
# b=func(a['1'],a[i],a[i+1],c[2],c[0],c[1])
b=func(a['1'],a[i],a[i+1],c[2],c['1'],c[0],c[1])
# b=func(a,b,c[1])
self.ds_opt_adam.adam_update_copy(self.opt_id, state['step'], group['lr'], beta1, beta2, group['eps'], group['weight_decay'], group['bias_correction'], p.data, p.grad.data, state['exp_avg'], state['exp_avg_sq'], fp16_param_groups[group_id][param_id].data)
'''
    # 对找到的代码片段不断进行转换直至不可以转换
    code_list=[
        ["user_check = self.run_function('file.get_user', [check])","mode_check = self.run_function('file.get_mode', [check])"],
        ["name = obj.find('name').text.strip()","bbox = obj.find('bndbox')"],
        ["self._tmp_path = tmp_path","self._config = config","self.location = str(tmp_path)"]]
    # while 1:
    save_complicated_code_dir = util.data_root + "complicated_code_dir/var_unpack_func_call_only_same_dengcha_subscript_complicated_pkl/"
    for file_name in os.listdir(save_complicated_code_dir):
        complicate_code = util.load_pkl(save_complicated_code_dir, file_name[:-4])
        for each_file in complicate_code:
            # for code_list, file_path,file_html in each_file:
            #
            #     print("count: ",code_list)
            code_map = each_file[0]
            for node, item in code_map.items():
                print(">>>>>>>old code1: ", ast.unparse(node))
                transform_var_unpack_call(node, item)
                break


    # test_save_complicated_code_dir = util.data_root + "test_complicated_code_dir/"
    # file_name = "var_unpack_func_call_only_same_dengcha_subscript_complicated"
    # map_code=util.load_pkl(test_save_complicated_code_dir,file_name)
    # for node, item in map_code.items():
    #     print(">>>>>>>old code1: ", ast.unparse(node))
    #     transform_var_unpack_call(node,item)



















import ast, copy
import sys,os
# sys.path.append("/Users/zhangzejunzhangzejun/PycharmProjects/pythonProject/python-ast-explorer-master/extract_transform_complicate_code_new/")
abs_path=os.path.abspath(os.path.dirname(__file__))

pack_path="/".join(abs_path.split("/")[:-2])
# print("pack_path: ",pack_path)
sys.path.append(pack_path)
sys.path.append(pack_path+"/RefactoringIdioms")
# print("pack_path_2: ",pack_path+"/RefactoringIdioms")
# import time
# from code1 import util
# from extract_simp_cmpl_data import ast_util
# from extract_simp_cmpl_data.extract_compli_truth_value_test_code import decide_compare_complicate_truth_value
# from code1.transform_c_s.transform_truth_value_test_compli_to_simple import transform_c_s_truth_value_test
# from pathos.multiprocessing import ProcessingPool as newPool
from RefactoringIdioms.extract_simp_cmpl_data import  ast_util
from RefactoringIdioms.extract_simp_cmpl_data.extract_compli_truth_value_test_code import decide_compare_complicate_truth_value
from RefactoringIdioms.transform_c_s.transform_truth_value_test_compli_to_simple import transform_c_s_truth_value_test
# from pathos.multiprocessing import ProcessingPool as newPool
import traceback
# from code1.extract_simp_cmpl_data import ast_util
# from code1.extract_simp_cmpl_data.extract_compli_truth_value_test_code import decide_compare_complicate_truth_value
# from pathos.multiprocessing import ProcessingPool as newPool
# from code1.transform_c_s.transform_truth_value_test_compli_to_simple import transform_c_s_truth_value_test

'''
这里在遍历的时候需要记录下 变量T的读写
广度优先遍历
input: 代码片段
对于当前层
if 是赋值语句 (记录 左边的write写的变量T)
    if 后面节点是 for节点
        if for节点的下一个节点是if 语句，其中test中存在T变量
            获得for节点的body
            if exist 涉及T变量的赋值语句
               后面节点是否存在 break节点
    else:
       判断是否T有读操作()

'''


# def traverse_for_else_comp(node,dict_source_code,file_path=None):
#     if
#     if isinstance(node, ast.For):
#
#     pass
class Analyzer(ast.NodeVisitor):
    def __init__(self,class_name="NULL"):
        self.func_def_list = []
        self.class_name = class_name
    def visit_FunctionDef(self, node):
        self.func_def_list.append([node, self.class_name])

        # print("fun name: ", node.name)

    def visit_ClassDef(self, node):

        class_ana=Analyzer(node.name)
        for stmt in node.body:
            class_ana.visit(stmt)
        self.func_def_list.extend(class_ana.func_def_list)
        # print("class name: ",class_ana.class_name)
    #
    def visit_If(self, node: ast.If):
        if ast.unparse(node.test) == "__name__ == '__main__'":
            self.func_def_list.append([node,"NULL"])


def get_intersect_vars(assign_list_in_for, vars_if_after_for):
    intersect_flag = 0
    intersect_infor = []

    for ind, node in enumerate(assign_list_in_for):
        if isinstance(node, (ast.Assign,ast.AnnAssign)):
            # print("come assign node: ",node.targets[0].__dict__)
            vars_assign_one = []
            intersect_vars = set([])
            targets=node.targets if isinstance(node,ast.Assign) else [node.target]
            for target in targets:
                vars_assign_one.append(ast.unparse(target))
                # visit_vars(target, vars_assign_one)
            intersect_vars |= set(vars_assign_one) & set(vars_if_after_for)
            if intersect_vars:
                intersect_flag = 1
                intersect_infor.append([node.lineno, node, intersect_vars])

    return intersect_flag, intersect_infor


def whether_is_break(node, if_vars_list):
    # print("the node is: ", node,node.__dict__)
    break_flag = 0
    # if isinstance(node, list):
    #     print("come here")
    #     for e in node:
    #         break_flag=break_flag or whether_is_break(e)
    # else:
    if isinstance(node, (ast.For, ast.While)):
        return 0
    elif isinstance(node, ast.Break):
        return 1
    elif isinstance(node, ast.Assign):
        pass
    elif isinstance(node.ast.If):
        pass
    else:
        for e in ast.iter_child_nodes(node):
            break_flag = break_flag or whether_is_break(e)
    return break_flag


def whether_contain_break_and_const_assign(node, assign_list_in_for, if_list_in_for,break_list_in_for):
    # assign_list_in_for=[]
    # if_list_in_for=[]
    break_flag = 0
    assign_list = []
    if hasattr(node, "body"):
        # print(">>>>>>the node: ",ast.unparse(node))
        for ind, e in enumerate(node.body):

            if isinstance(e, ast.Break):
                    break_list_in_for.append(e)
                    break_flag=1

                    # if 1:#isinstance(node, ast.If):
                    if_list_in_for.append(node)
                    assign_list_in_for.append(assign_list)

                # break
            elif isinstance(e, (ast.Assign,ast.AnnAssign,ast.AugAssign)):
                assign_list.append(e)
            elif isinstance(e, (ast.For, ast.While,ast.AsyncFor)):
                continue

            else:
                # print(">>>>>>the ele: ", ast.unparse(e))
                break_flag = whether_contain_break_and_const_assign(e, assign_list_in_for, if_list_in_for,break_list_in_for) or break_flag
    if hasattr(node,"orelse"):

        for e in node.orelse:
            if isinstance(e, ast.Break):
                    break_list_in_for.append(e)
                    break_flag=1

                    # if 1:#isinstance(node, ast.If):
                    if_list_in_for.append(node)
                    assign_list_in_for.append(assign_list)

                # break
            elif isinstance(e, (ast.Assign,ast.AnnAssign,ast.AugAssign)):
                assign_list.append(e)
            elif isinstance(e, (ast.For, ast.While,ast.AsyncFor)):
                continue

            else:
                # print(">>>>>>the ele: ", ast.unparse(e))
                break_flag = whether_contain_break_and_const_assign(e, assign_list_in_for, if_list_in_for,break_list_in_for) or break_flag

            # break_flag = whether_contain_break_and_const_assign(e_e, assign_list_in_for, if_list_in_for,
            #                                                     break_list_in_for) or break_flag

    return break_flag


def visit_vars(target, list_vars):
    # print(">>>>>>>target: ",target.__dict__)
    if isinstance(target, (ast.Name, ast.Subscript, ast.Attribute)):
        list_vars.append(ast.unparse(target))
    else:
        for e in ast.iter_child_nodes(target):
            visit_vars(e, list_vars)


def whether_if_and_contain_var(node):
    flag_if_var = 0
    if_vars_list = None
    if isinstance(node, ast.If):
        flag_if_var = 1
        if_vars_list = node
        # visit_vars(node.test, if_vars_list)
        return flag_if_var, if_vars_list
    else:
        return flag_if_var, if_vars_list


def transform_truth_value_test(test):
    if isinstance(test, ast.Compare):
        if decide_compare_complicate_truth_value(test):
            return transform_c_s_truth_value_test(test)

    return test


def is_same_two_if_test(test_1, test_2):
    def is_same_ops(op1, op2):
        if isinstance(op1, (ast.Eq, ast.Is)) and isinstance(op2, (ast.Eq, ast.Is)):
            return 1
        elif isinstance(op2, (ast.NotEq, ast.IsNot)) and isinstance(op1, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op1, (ast.Lt)) and isinstance(op2, (ast.Lt)):
            return 1
        elif isinstance(op1, (ast.LtE)) and isinstance(op2, (ast.LtE)):
            return 1
        elif isinstance(op1, (ast.Gt)) and isinstance(op2, (ast.Gt)):
            return 1
        elif isinstance(op1, (ast.GtE)) and isinstance(op2, (ast.GtE)):
            return 1
        return 0

    test_1 = transform_truth_value_test(test_1)
    test_2 = transform_truth_value_test(test_2)
    # print("is_same_two_if_test: ",ast.unparse(test_2),ast.unparse(test_1))
    if isinstance(test_1, ast.Compare) and isinstance(test_2, ast.Compare):
        op1 = test_1.ops[0]
        operators_1 = set([ast.unparse(test_1.left), ast.unparse(test_1.comparators[0])])
        op2 = test_2.ops[0]
        operators_2 = set([ast.unparse(test_2.left), ast.unparse(test_2.comparators[0])])
        # print("compare two ops: ",is_differ_ops(op1, op2))
        if is_same_ops(op1, op2) and operators_1 == operators_2:
            return 1
        pass
    else:
        test_1_str = ast.unparse(test_1)
        test_2_str = ast.unparse(test_2)
        if test_1_str == test_2_str or test_1_str == test_2_str:
            return 1
    return 0


def is_differ_two_if_test(test_1, test_2):
    def is_differ_ops(op1, op2):
        if isinstance(op1, (ast.Eq, ast.Is)) and isinstance(op2, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op2, (ast.Eq, ast.Is)) and isinstance(op1, (ast.NotEq, ast.IsNot)):
            return 1
        elif isinstance(op1, (ast.Lt)) and isinstance(op2, (ast.GtE)):
            return 1
        elif isinstance(op1, (ast.LtE)) and isinstance(op2, (ast.Gt)):
            return 1
        elif isinstance(op1, (ast.Gt)) and isinstance(op2, (ast.LtE)):
            return 1
        elif isinstance(op1, (ast.GtE)) and isinstance(op2, (ast.Lt)):
            return 1
        return 0

    test_1 = transform_truth_value_test(test_1)
    test_2 = transform_truth_value_test(test_2)

    if isinstance(test_1, ast.Compare) and isinstance(test_2, ast.Compare):
        # print("compare: ",ast.unparse(test_1),ast.unparse(test_2))
        if len(test_1.ops) > 1 and len(test_2.ops) > 1:
            return 0
        op1 = test_1.ops[0]
        operators_1 = set([ast.unparse(test_1.left), ast.unparse(test_1.comparators[0])])
        op2 = test_2.ops[0]
        operators_2 = set([ast.unparse(test_2.left), ast.unparse(test_2.comparators[0])])
        # print("compare two ops: ",is_differ_ops(op1, op2))
        if is_differ_ops(op1, op2) and operators_1 == operators_2:
            return 1
    else:
        test_1_str = ast.unparse(test_1)
        test_2_str = ast.unparse(test_2)
        if "not " + test_1_str == test_2_str or test_1_str == "not " + test_2_str:
            return 1

    return 0


def is_same_ass_init_if_test(assign, test):
    count = 0
    targets = assign.targets if isinstance(assign, ast.Assign) else [assign.target]
    # targets = assign.targets
    value = assign.value
    for tar in targets:
        count += ast_util.get_basic_count(tar)
    if count > 1:
        return 0
    count = ast_util.get_basic_count(value)
    if count > 1:
        return 0
    tree = ast.parse(ast.unparse(targets[0]) + "==" + ast.unparse(value))
    new_test_ass = None
    for node in ast.walk(tree):
        if isinstance(node, ast.Compare):
            return is_same_two_if_test(node, test)

    return 0
    pass


def is_differ_ass_again_if_test(assign, test):
    count = 0
    targets = assign.targets if isinstance(assign, ast.Assign) else [assign.target]
    # targets = assign.targets
    value = assign.value
    for tar in targets:
        count += ast_util.get_basic_count(tar)
    if count > 1:
        return 0
    count = ast_util.get_basic_count(value)
    if count > 1:
        return 0
    tree = ast.parse(ast.unparse(targets[0]) + "==" + ast.unparse(value))

    for node in ast.walk(tree):
        if isinstance(node, ast.Compare):
            # print("is_differ_ass_again_if_test: ",ast.unparse(node),ast.unparse(test))
            return is_differ_two_if_test(node, test)

    return 0
def modify_normalize(tree, child, if_varnode, if_list_in_for, orelse_node,stmts_insert_break):
    child.orelse = orelse_node
    len_body = len(if_list_in_for[0].body)
    for ind_e_if, e_if in enumerate(stmts_insert_break):
        if_list_in_for[0].body.insert(len_body - 1 + ind_e_if, e_if)
    tree.body.remove(if_varnode)
def modify(tree, child, if_varnode, if_list_in_for, flag=0):
    if flag == 0:
        child.orelse = if_varnode.orelse
        len_body = len(if_list_in_for.body)

        for ind_e_if, e_if in enumerate(if_varnode.body):

            if isinstance(e_if, ast.Break):

                return 1
            if_list_in_for.body.insert(len_body - 1 + ind_e_if, e_if)
        if if_varnode in tree.body:

            tree.body.remove(if_varnode)

    elif flag == 1:
        child.orelse = if_varnode.body
        len_body = len(if_list_in_for.body)
        # break_flag=0
        if if_varnode.orelse:
            for ind_e_if, e_if in enumerate(if_varnode.orelse):
                if isinstance(e_if, ast.Continue):
                    return 1
                if isinstance(e_if,ast.Break):
                    return 1
                if_list_in_for.body.insert(len_body - 1 + ind_e_if, e_if)
        if if_varnode in tree.body:
            # if break_flag:
            #     tree.body.insert(tree.body.index(if_varnode),ast.Break())
            tree.body.remove(if_varnode)


def remove_assign(tree_copy, child_copy, tree, child, if_vars_list, intersect_infor_ass_list_in_for,
                      intersect_infor_ass_init,ind):
    # print("come remove_assign: ",len(intersect_infor_ass_init))
    init_ass_remove=0
    if 1:
        # print("len intersect_infor_ass_list_in_for: ",len(intersect_infor_ass_list_in_for))
        count = 0
        for node in ast.walk(child):
            if ast.unparse(node) == if_vars_list[0]:
                count += 1
                if count > len(intersect_infor_ass_list_in_for):
                    # print("count: ",count,len(intersect_infor_ass_list_in_for))
                    break
        else:
            # print("come remove_assign: code1 behind should not occur var")
            count = 0
            for body in tree.body[ind + 1:]:
                for node in ast.walk(body):
                    if ast.unparse(node) == if_vars_list[0]:
                        count += 1
                        if count > 1:
                            break
                if count > 1:
                    break
            else:

                for e in ast.walk(child_copy):
                    if hasattr(e, "body") and isinstance(e.body,list):
                        # print("body: ",e,ast.unparse(e))
                        for e_each in e.body:
                            for ass_each_list in intersect_infor_ass_list_in_for:
                                # print("ass_each_list: ",ass_each_list)
                                if e_each == ass_each_list[0][1]:

                                    e.body.remove(e_each)
                    if hasattr(e,"orelse"):
                        for e_e in e.orelse:
                            if e_e == ass_each_list[0][1]:
                                e.orelse.remove(e_e)
                                break

                for e in ast.walk(tree_copy):
                    if hasattr(e, "body")  and isinstance(e.body,list):
                        # print("body: ",e,ast.unparse(e))
                        for node in e.body:
                            if ast.unparse(intersect_infor_ass_init[0][1]) == ast.unparse(node):
                                e.body.remove(node)
                                init_ass_remove=1
                                break
                        else:
                            continue
                        break
                    if hasattr(e, "orelse"):
                            for e_e in e.orelse:
                                if e_e == ass_each_list[0][1]:
                                    e.orelse.remove(e_e)
                                    break
    # print("new_tree: ", ast.unparse(tree))
    return init_ass_remove
def traverse_cur_layer(tree, code_list, ass_init_list):
    # print(">>>>>>>>>assign_list: ", ass_init_list)
    if hasattr(tree, "body"):
        for ind,child in enumerate(tree.body):
            tree_copy = copy.deepcopy(tree)
            child_copy = tree_copy.body[ind]
            if isinstance(child, (ast.For, ast.While)):
                if child.orelse:
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                if ind + 1 >= len(tree.body):
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                if_vars_list, intersect_infor, assign_list_in_for = [], [], []
                # print("tree_copy: ",tree_copy,tree.body,ast.unparse(child),child)
                next_child = tree_copy.body[ind + 1]

                flag_if_var, if_varnode = whether_if_and_contain_var(next_child)

                # ignore this code1 if a and b or c:
                if not if_varnode or (isinstance(if_varnode,ast.BoolOp) and isinstance(if_varnode.op,(ast.And,ast.Or))):
                    traverse_cur_layer(child, code_list, ass_init_list)
                    continue
                # print("it is if node after for stmt: ", flag_if_var, ast.unparse(next_child), if_vars_list)
                all_assign_list_in_for = []
                if_list_in_for = []
                break_list_in_for=[]
                break_flag = whether_contain_break_and_const_assign(child_copy, all_assign_list_in_for, if_list_in_for,break_list_in_for)
                flag_simplify=1
                assign_flag=1
                if break_flag:
                    # # print(">>>>come break: ",len(if_list_in_for))
                    # for ass in ass_init_list:
                    #     print("ass_init: ",ast.unparse(ass))
                    intersect_infor_ass_list_in_for=[]
                    visit_vars(if_varnode.test, if_vars_list)
                    intersect_flag_ass_init, intersect_infor_ass_init = get_intersect_vars(ass_init_list, if_vars_list)
                    # init_list没有初始化的语句
                    # for ass in intersect_infor_ass_init:
                    #     print("intersect ass_init: ",ast.unparse(ass[1]))
                    if not intersect_flag_ass_init or len(intersect_infor_ass_init[-1][-1]) > 1:
                        flag_simplify = 0
                        break
                    else:
                        #ass_init的语句应该和for节点同父母或者for节点属于ass_init.parent
                        for ind_chi, ass_child in enumerate(tree.body[:ind]):
                            if isinstance(ass_child,(ast.Assign,ast.AnnAssign)):
                                continue
                            else:
                                for ass_child_child in ast.walk(ass_child):
                                    if isinstance(ass_child_child,(ast.Assign,ast.AnnAssign,ast.AugAssign)):
                                        for ass_init_node in intersect_infor_ass_init:
                                            if ass_child_child.lineno == ass_init_node[1].lineno:
                                                flag_simplify=0
                                                break
                                        else:
                                            continue
                                        flag_simplify = 0
                                        break
                                else:
                                    continue
                                flag_simplify = 0
                                break

                        if not flag_simplify:
                            traverse_cur_layer(child, code_list, ass_init_list)
                            print("init ass is not valid")
                            continue



                        # if contain_other_ass(intersect_infor_ass_init):
                        #     flag_simplify = 0
                        #     break
                        if is_same_ass_init_if_test(intersect_infor_ass_init[-1][1], if_varnode.test):
                            # print("yes  if_test and assign init could be simplified to for else: ",ast.unparse(tree_copy))
                            # print("child copy: ",ast.unparse(child_copy),"-------\n",ast.unparse(if_varnode),if_list_in_for)

                            # modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                            # print("assign init new_tree: ",ast.unparse(tree_copy))
                            flag_simplify = 1
                            assign_flag = 1
                        elif if_varnode.orelse and is_differ_ass_again_if_test(intersect_infor_ass_init[-1][1],
                                                                               if_varnode.test):
                            # for if_list_each in if_list_in_for:
                            #     modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                            flag_simplify = 1
                            assign_flag = 1
                        else:
                            flag_simplify = 0

                    if not flag_simplify:
                        traverse_cur_layer(child, code_list, ass_init_list)
                        continue
                    # print(">>>>has determined ass_init and if_var_test: ", len(if_list_in_for))
                    for ind_test,assign_list_in_for in enumerate(all_assign_list_in_for):
                        if_list_each=if_list_in_for[ind_test]
                        flag_inn, info_inn = get_intersect_vars(assign_list_in_for, if_vars_list)

                        if flag_inn:
                            # print("all_assign_list_in_for: ", len(all_assign_list_in_for),len(if_list_in_for),len(break_list_in_for))
                            # for e in info_inn:
                            #     print("intersect: ",ast.unparse(e[1]))
                            intersect_infor_ass_list_in_for.append(info_inn)
                            for line_no,ass,inter_var in info_inn:
                                # if ass_init_list and ast.unparse(ass_init_list[-1].value) != ast.unparse(ass.value):
                                    if is_differ_ass_again_if_test(ass, if_varnode.test):
                                        body_break_flag=modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                                        if body_break_flag:
                                            flag_simplify = 0
                                        break
                                    elif if_varnode.orelse and is_same_ass_init_if_test(ass, if_varnode.test):
                                        body_break_flag=modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                                        if body_break_flag:
                                            flag_simplify = 0
                                        break
                            else:
                                flag_simplify = 0
                                break
                        else:
                            if hasattr(if_list_each, "test"):

                                if is_differ_two_if_test(if_varnode.test, if_list_each.test):
                                    body_break_flag=modify(tree_copy, child_copy, if_varnode, if_list_each, 1)
                                    if body_break_flag:
                                        flag_simplify = 0
                                    assign_flag = 0
                                elif if_varnode.orelse:
                                    if if_list_each.test and is_same_two_if_test(
                                        if_varnode.test, if_list_each.test):
                                        body_break_flag=modify(tree_copy, child_copy, if_varnode, if_list_each, 0)
                                        if body_break_flag:
                                            flag_simplify = 0
                                        assign_flag=0
                                        pass
                                    else:
                                        flag_simplify = 0
                                        break

                                else:
                                    flag_simplify = 0
                                    break
                            else:
                                flag_simplify = 0
                                break

                    init_ass_remove_flag=0
                    # print(">>>>>>>flag_simplify,assign_flag",flag_simplify,assign_flag,child_copy.lineno)
                    if flag_simplify and assign_flag:
                            init_ass_remove_flag=remove_assign(tree_copy,child_copy,tree,child,if_vars_list,intersect_infor_ass_list_in_for,intersect_infor_ass_init,ind)
                    # print(">>>>>>>>remove assign flag: ", assign_flag,init_ass_remove_flag)


                    if flag_simplify:
                        # print(">>>>>>>>>>>>>old tree:\n",ast.unparse(child_copy),child_copy.lineno,ast.unparse(tree_copy))
                        # print(">>>>>>>>>>>>>new tree:\n", ast.unparse(child))#intersect_infor_ass_init[0][1]
                        code_list.append([tree,tree_copy,break_list_in_for,child,child_copy,intersect_infor_ass_init[0][1],if_varnode,init_ass_remove_flag])
                        # code_list.append([ ast.unparse(tree),ast.unparse(tree_copy),break_list_in_for[0].lineno])
                        # code_list.append(
                        #     [assign_init + "\n" + ast.unparse(child) + "\n" + ast.unparse(next_child), child.lineno,
                        #      next_child.lineno])


            else:
                # for e_chi in ast.walk(child_copy):
                    if isinstance(child_copy, (ast.Assign,ast.AnnAssign)) and child_copy not in ass_init_list  :
                        # print("else: ",ast.unparse(e_chi))
                        ass_init_list.append(child_copy)
                # print("assign_list: ",ass_init_list)

            traverse_cur_layer(child, code_list, ass_init_list)
def transform_for_else_code(content):
    code_pair_list = []
    # print(content)
    try:

        file_tree = ast.parse(content)
        ana_py = ast_util.Fun_Analyzer()
        ana_py.visit(file_tree)

        for tree, class_name in ana_py.func_def_list:
            try:
                me_name=tree.name
            except:
                me_name=""
            new_code_list = []
            ass_init_list = []
            traverse_cur_layer(tree, new_code_list, ass_init_list)
            # #result_compli_for_else_list.append(
            #                     [repo_name, file_html, cl,me,code[3].lineno,ast.unparse(code[5])+"\n"+ast.unparse(code[3])+"\n"+ast.unparse(code[6]), ast.unparse(code[4]),code[7]])

                # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # assign_list[ind_ass].append(new_code)
                # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
            for code in new_code_list:
                old_code_5_content = ast.get_source_segment(content, code[5])
                old_code_3_content = ast.get_source_segment(content, code[3])
                old_code_6_content = ast.get_source_segment(content, code[6])
                old_node_space_prefix=old_code_3_content.split('\n')[1]

                new_code_fragment_list=ast.unparse(code[4]).split('\n')
                for ind,e_fragment in enumerate(new_code_fragment_list):
                    if ind>0:
                        new_code_fragment_list[ind] = ' ' * code[3].col_offset + e_fragment

                if code[7]:
                    line_list=[[(code[5].lineno,code[5].col_offset), (code[5].end_lineno,code[5].end_col_offset)]]
                    line_list.append([(code[3].lineno,code[3].col_offset), (code[3].end_lineno,code[3].end_col_offset)])
                    line_list.append([(code[6].lineno,code[6].col_offset), (code[6].end_lineno,code[6].end_col_offset)])


                    # code_pair_list.append([class_name,me_name,[old_code_5_content,old_code_3_content,old_code_6_content],  ['',ast.unparse(code[4]),''],line_list])
                    code_pair_list.append([class_name,me_name,[old_code_5_content,old_code_3_content,old_code_6_content],  ['',"\n".join(new_code_fragment_list),''],line_list])

                else:
                    line_list = []
                    line_list.append(
                        [(code[3].lineno, code[3].col_offset), (code[3].end_lineno, code[3].end_col_offset)])
                    line_list.append(
                        [(code[6].lineno, code[6].col_offset), (code[6].end_lineno, code[6].end_col_offset)])
                    # code_pair_list.append([class_name,me_name,[old_code_3_content,old_code_6_content], [ast.unparse(code[5]),ast.unparse(code[4])],line_list])
                    code_pair_list.append([class_name,me_name,[old_code_3_content,old_code_6_content], ["\n".join(new_code_fragment_list),''],line_list])

                # code_pair_list.append([class_name, me_name,
                #                        ast.unparse(code[5]) + "\n" + ast.unparse(code[3]) + "\n" + ast.unparse(code[6]),
                #                        ast.unparse(code[4]) if code[7] else ast.unparse(code[5]) + "\n" + ast.unparse(
                #                            code[4]), line_list])

        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

def transform_part_for_else_code(content):
    code_pair_list = []
    # print(content)
    try:


        tree = ast.parse(content)
        if 1:
        # for tree, class_name in ana_py.func_def_list:
            try:
                me_name=tree.name
            except:
                me_name=""
            new_code_list = []
            ass_init_list = []
            traverse_cur_layer(tree, new_code_list, ass_init_list)
            # #result_compli_for_else_list.append(
            #                     [repo_name, file_html, cl,me,code[3].lineno,ast.unparse(code[5])+"\n"+ast.unparse(code[3])+"\n"+ast.unparse(code[6]), ast.unparse(code[4]),code[7]])

                # new_file_content = replace_file_content_ass(content, each_assign_list, new_code)
                # assign_list[ind_ass].append(new_code)
                # complic_code_me_info_dir_pkl = util.data_root + "complic_code_me_info_dir_pkl/each_idiom_type_all_methods/multi_ass/"  # for_else
                # util.save_file(complic_code_me_info_dir_pkl, "test"+str(ind_ass), new_file_content, ".txt", "w")
                # util.save_file(complic_code_me_info_dir_pkl, "test_old"+str(ind_ass), content, ".txt", "w")
            for code in new_code_list:
                old_code_5_content = ast.get_source_segment(content, code[5])
                old_code_3_content = ast.get_source_segment(content, code[3])
                old_code_6_content = ast.get_source_segment(content, code[6])
                old_node_space_prefix=old_code_3_content.split('\n')[1]

                new_code_fragment_list=ast.unparse(code[4]).split('\n')
                for ind,e_fragment in enumerate(new_code_fragment_list):
                    if ind>0:
                        new_code_fragment_list[ind] = ' ' * code[3].col_offset + e_fragment

                if code[7]:
                    line_list=[[(code[5].lineno,code[5].col_offset), (code[5].end_lineno,code[5].end_col_offset)]]
                    line_list.append([(code[3].lineno,code[3].col_offset), (code[3].end_lineno,code[3].end_col_offset)])
                    line_list.append([(code[6].lineno,code[6].col_offset), (code[6].end_lineno,code[6].end_col_offset)])


                    # code_pair_list.append([class_name,me_name,[old_code_5_content,old_code_3_content,old_code_6_content],  ['',ast.unparse(code[4]),''],line_list])
                    code_pair_list.append(['',me_name,[old_code_5_content,old_code_3_content,old_code_6_content],  ['',"\n".join(new_code_fragment_list),''],line_list])

                else:
                    line_list = []
                    line_list.append(
                        [(code[3].lineno, code[3].col_offset), (code[3].end_lineno, code[3].end_col_offset)])
                    line_list.append(
                        [(code[6].lineno, code[6].col_offset), (code[6].end_lineno, code[6].end_col_offset)])
                    # code_pair_list.append([class_name,me_name,[old_code_3_content,old_code_6_content], [ast.unparse(code[5]),ast.unparse(code[4])],line_list])
                    code_pair_list.append(['',me_name,[old_code_3_content,old_code_6_content], ["\n".join(new_code_fragment_list),''],line_list])

                # code_pair_list.append([class_name, me_name,
                #                        ast.unparse(code[5]) + "\n" + ast.unparse(code[3]) + "\n" + ast.unparse(code[6]),
                #                        ast.unparse(code[4]) if code[7] else ast.unparse(code[5]) + "\n" + ast.unparse(
                #                            code[4]), line_list])

        return code_pair_list

    except:
        traceback.print_exc()
        return code_pair_list

# def save_repo_for_else_complicated(repo_name):
#     start=time.time()
#     count_complicated_code = 0
#     print("come the repo: ", repo_name)
#     one_repo_for_else_code_list = []
#
#     dict_file=dict()
#     # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
#     #     print("the repo has been saved before")
#     #     return None
#
#     for ind,file_info in enumerate(dict_repo_file_python[repo_name]):
#
#
#
#         file_path = file_info["file_path"]
#         # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
#         #     continue
#         file_html = file_info["file_html"]
#         # if file_html!="https://github.com/ansible/ansible-modules-core/tree/master/cloud/openstack/os_router.py":#"https://github.com/KichangKim/DeepDanbooru/tree/master/deepdanbooru/commands/download_tags.py":#"https://github.com/googleworkspace/python-samples/tree/master/classroom/snippets/classroom_snippets.py":#"https://github.com/sympy/sympy/tree/master/sympy/utilities/enumerative.py":#"https://github.com/sympy/sympy/tree/master/sympy/tensor/array/expressions/conv_array_to_matrix.py":#"https://github.com/sympy/sympy/tree/master/sympy/testing/quality_unicode.py":#"#
#         #     continue
#
#
#         try:
#             content = util.load_file_path(file_path)
#         except:
#             print(f"{file_path} is not existed!")
#             continue
#
#         # print("content: ",content)
#         try:
#             file_tree = ast.parse(content)
#             ana_py = ast_util.Fun_Analyzer()
#             ana_py.visit(file_tree)
#
#             # dict_file["repo_name"]=repo_name
#
#
#             dict_class=dict()
#             for tree, class_name in ana_py.func_def_list:
#
#                 # print("tree_ func_name",tree.__dict__)
#                 new_code_list = []
#                 ass_init_list = []
#                 traverse_cur_layer(tree, new_code_list, ass_init_list)
#                 ast_util.set_dict_class_code_list(tree,dict_class, class_name, new_code_list)
#
#
#             dict_file[file_html]=dict_class
#
#             # if code_list:
#             #     one_repo_for_else_code_list.append([code_list, file_path, file_html])
#                 # if len(one_repo_for_else_code_list)>2:
#                 #     break
#
#         except SyntaxError:
#             print("the file has syntax error")
#             continue
#         except ValueError:
#             traceback.print_exc()
#             print("the file has value error: ", file_html)
#             continue
#         except:
#             print("the file error: ", file_html)
#             traceback.print_exc()
#         # break
#     end = time.time()
#     #'''
#     if dict_file:
#         count_complicated_code = count_complicated_code + len(one_repo_for_else_code_list)
#         # print("it exists for else complicated code1: ", len(one_repo_for_else_code_list))
#         util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#         print(end-start," save successfully! ", save_complicated_code_dir_pkl + repo_name)
#     else:
#         print(end-start," the repo has no for else")
#         util.save_pkl(save_complicated_code_dir_pkl, repo_name, dict_file)
#         # util.save_json(save_complicated_code_dir, repo_name, one_repo_for_else_code_list)
#     #'''
#     return count_complicated_code


if __name__ == '__main__':
    pass
#     code = '''
# # if match:
# #     skip = True
# #     for pat in self.patterns:
# #         if not pat.exclusion:
# #             continue
# #         if pat.cleaned_pattern.startswith(normalize_slashes(fpath)):
# #             skip = False
# #             break
# #     if skip:
# #         continue
#
# # for current_package in self._current_packages:
# #     found = False
# #     for (result_package, _) in self._result_packages:
# #         if current_package.name == result_package.name:
# #             found = True
# #             break
# #     if not found:
# #         for installed_package in self._installed_packages:
# #             if installed_package.name == current_package.name:
# #                 operations.append(Uninstall(current_package))
# # def __get_path_from_ref(self, ref):
# #     refs = ref.split('/')
# #     for (i, ref) in enumerate(refs):
# #         if ref.endswith('.kp'):
# #             break
# #     if not ref.endswith('.kp'):
# #         return None
# #     return '/'.join(refs[:i + 1])
#
# # crumb = None
# # sess = requests.Session()
# # for i in range(self.retries + 1):
# #         resp = sess.get(url, **sesskwargs)
# #         if resp.status_code != requests.codes.ok:
# #             continue
# #         txt = resp.text
# #         i = txt.find('CrumbStore')
# #         istart = txt.find('"', i + len('crumb') + 1)
# #         if istart == -1:
# #             continue
# #         istart += 1
# #         iend = txt.find('"', istart)
# #         if iend == -1:
# #             continue
# #         crumb = txt[istart:iend]
# #         crumb = crumb.encode('ascii').decode('unicode-escape')
# #         break
# # if crumb is None:
# #         self.error = 'Crumb not found'
# #         self.f = None
# #         return
#
#
# # is_choice = False
# # if len(choice_list) == 0:
# #     is_choice = True
# # for ch in choice_list:
# #     if ch in title:
# #         is_choice = True
# #         break
# # if is_choice == False:
# #     return False
# # is_ban = False
# # for ba in ban_list:
# #     if ba in title:
# #         is_ban = True
# #         break
# # if is_ban == True:
# #     return False
# # return True
#
# # def trigger(self, char, *args, **kwargs):
# #     if isinstance(char, six.string_types) and len(char) == 1:
# #         char = ord(char)
# #     func = None
# #     for controller in self.parents:
# #         func = controller.character_map.get((self.last_char, char))
# #         if func:
# #             break
# #         func = controller.character_map.get(char)
# #         if func:
# #             break
# #     if func:
# #         self.last_char = None
# #         return func(self.instance, *args, **kwargs)
# #     else:
# #         self.last_char = char
# #         return None
#
# root_hierarchical_hash = None
# if hierarchical_hashes:
#     hierarchical_grouphashes = {h.hash: h for h in GroupHash.objects.filter(project=project, hash__in=hierarchical_hashes)}
#     for hash in reversed(hierarchical_hashes):
#         group_hash = hierarchical_grouphashes.get(hash)
#         if group_hash is not None and group_hash.state == GroupHash.State.SPLIT:
#             found_split = True
#             break
#         root_hierarchical_hash = hash
#         if group_hash is not None:
#             all_grouphashes.append(group_hash)
#     if root_hierarchical_hash is None:
#         root_hierarchical_hash = hierarchical_hashes[-1]
#         group_hash = hierarchical_grouphashes.get(root_hierarchical_hash)
#         if group_hash is not None:
#             all_grouphashes.append(group_hash)
# # for i in range(2):
# #     print(i)
# #     break
# # if a:
# #     print("1")
#
# # response = None
# # callback, callback_args, callback_kwargs = self.resolve_request(request)
# #
# # # Apply view middleware.
# # for middleware_method in self._view_middleware:
# #     response = await middleware_method(request, callback, callback_args, callback_kwargs)
# #     if response:
# #         break
# #
# # if response is None:
# #     # print("test_if")
# #     wrapped_callback = self.make_view_atomic(callback)
# # else:
# #     print("test")
# #     print("test2")
#
#
#
# # _checkpoint_callback = None
# # require_checkpoint = False
# #
# # with remote_store.get_local_output_dir() as run_output_dir:
# #     logs_path = os.path.join(run_output_dir, remote_store.logs_subdir)
# #     os.makedirs(logs_path, exist_ok=True)
# #     print(f"Made directory {logs_path} for horovod rank {hvd.rank()}")
# #     ckpt_dir = run_output_dir
# #     ckpt_filename = remote_store.checkpoint_filename
# #
# #     if logger is None:
# #         # Use default logger if no logger is supplied
# #         train_logger = TensorBoardLogger(logs_path)
# #         print(f"Setup logger: Using TensorBoardLogger: {train_logger}")
# #
# #     elif isinstance(logger, CometLogger) and logger._experiment_key is None:
# #         # Resume logger experiment key if passed correctly from CPU.
# #         train_logger = CometLogger(
# #             save_dir=logs_path,
# #             api_key=logger.api_key,
# #             experiment_key=logger_experiment_key,
# #         )
# #
# #         print(f"Setup logger: Resume comet logger: {vars(train_logger)}")
# #     else:
# #         # use logger passed in.
# #         train_logger = logger
# #         train_logger.save_dir = logs_path
# #         print(f"Setup logger: Using logger passed from estimator: {train_logger}")
# #
# #     # Lightning requires to add checkpoint callbacks for all ranks.
# #     # Otherwise we are seeing hanging in training.
# #     for cb in callbacks:
# #         if isinstance(cb, ModelCheckpoint):
# #             cb.dirpath = ckpt_dir
# #             cb.filename = ckpt_filename
# #             _checkpoint_callback = cb
# #             require_checkpoint = True
# #             break
# #     if not _checkpoint_callback:
# #         # By default 'monitor'=None which saves a checkpoint only for the last epoch.
# #         _checkpoint_callback = ModelCheckpoint(dirpath=ckpt_dir,
# #                                                filename=ckpt_filename,
# #                                                verbose=True)
# #     else:
# #         print("the else 1")
# #         print("the else 2")
#
# #
# # for module_name in relevant_modules:
# #     relevant_keys = set([key for key in state_dict.keys() if key.startswith(module_name + '.')])
# #     if relevant_keys:
# #         state_dict = {key.replace(module_name + '.', '', 1): value for (key, value) in state_dict.items() if key in relevant_keys}
# #         found = True
# #         break
# # if not found:
# #     warnings.warn(f'{relevant_modules} was not found at top level of state_dict!', UserWarning)
# # else:
# #     print("the else 1")
# #     print("the else 2")
# # for new_loc in new_locs:
# #     new_loc_nes = [xx for xx in [(new_loc[0] + 1, new_loc[1]), (new_loc[0] - 1, new_loc[1]), (new_loc[0], new_loc[1] + 1), (new_loc[0], new_loc[1] - 1)] if xx[0] >= 0 and xx[0] < my_fpath_map.shape[0] and (xx[1] >= 0) and (xx[1] < my_fpath_map.shape[1])]
# #     if fpath_map is not None and np.sum([fpath_map[nlne[0], nlne[1]] for nlne in new_loc_nes]) != 0:
# #         break_flag = True
# #         break
# #     if my_npath_map[new_loc[0], new_loc[1]] != -1:
# #         continue
# #     if npath_map is not None and npath_map[new_loc[0], new_loc[1]] != edge_id:
# #         break_flag = True
# #         break
# #     fpath.append(new_loc)
# # if break_flag is True:
# #     break
#
#
# # for controller in self.parents:
# #     func = controller.character_map.get((self.last_char, char))
# #     if func:
# #         break
# #     func = controller.character_map.get(char)
# #     if func:
# #         break
# # if func:
# #     self.last_char = None
# #     return func(self.instance, *args, **kwargs)
# # else:
# #     self.last_char = char
# #     return None
# found = False
# for a in action_filter:
#     if action.startswith(a):
#         found = True
#         break
# if not found:
#     continue
#
# # for (host, slots) in host_list:
# #     slots = int(slots)
# #     if slots > lsf.LSFUtils.get_num_gpus():
# #         raise ValueError("Invalid host input, slot count for host '{host}:{slots}' is greater than number of GPUs per host '{gpus}'.".format(host=host, slots=slots, gpus=lsf.LSFUtils.get_num_gpus()))
# #     needed_slots = min(slots, remaining_slots)
# #     validated_list.append((host, needed_slots))
# #     remaining_slots -= needed_slots
# #     if remaining_slots == 1:
# #         break
# # if remaining_slots != 1:
# #     raise ValueError('Not enough slots on the hosts to fulfill the {slots} requested.'.format(slots=settings.num_proc))
#
# # for f in possible_font_names:
# #     pdf_text_font = pdf_text_fonts.get(f, None)
# #     if pdf_text_font is not None:
# #         font_key = f
# #         break
# # if pdf_text_font:
# #     font = self.pdf_base.copy_foreign(pdf_text_font)
# # a=1
# # b=2
# # c,d=1,2
# # for i in range(2):
# #     a=1
# #     if i>1:
# #         if i>2:
# #             a=3
# #         a=2
# #         break
# # if a==2:
# #     print(1)
# # print("1")
# # def a():
# #     return 1
# # a=2
# # for i in range(5):
# #     a=1
# #     break
# # if a:
# #     print("test")
# # for i in range(2):
# #     if i>2:
# #         a=1
# #         for j in range(3):
# #             if j>2:
# #                 break
# # if a:
# #     print("it should not a 可简化的for else代码")
# # if a>2:
# #     for i in range(2):
# #         break
# # elif a>3:
# #     print("test")
# # def func2():
# #     while start < len(chars):
# #         end = len(chars)
# #         cur_substr = None
# #         while start < end:
# #             substr = ''.join(chars[start:end])
# #             if start == 0:
# #                 substr = sentencepiece_prefix + substr
# #             if start > 0:
# #                 substr = prefix + substr
# #             if substr in vocab:
# #                 cur_substr = substr
# #                 break
# #             end -= 1
# #         if cur_substr is None:
# #             is_bad = True
# #             break
# #         sub_tokens.append(cur_substr)
# #         sub_pos.append((start, end))
# #         start = end
# #     if is_bad:
# #         return ([unk_token], [(0, len(chars))])
# #     else:
# #         return (sub_tokens, sub_pos)
# # def func():
# #     for nexttag in tag_iter:
# #         tagname = nexttag.tag
# #         attributes = nexttag.attributes
# #         if tagname == 'a' and (nexttag.tag_type == CLOSE_TAG or (attributes.get('href') and (not attributes.get('href', '').startswith('#')))):
# #             if astart:
# #                 yield mklink(ahref, body[astart:nexttag.start], nofollow)
# #                 astart = ahref = None
# #                 nofollow = False
# #             href = attributes.get('href')
# #             if href:
# #                 ahref = href
# #                 astart = nexttag.end
# #                 nofollow = attributes.get('rel') == u'nofollow'
# #         elif tagname == 'head':
# #             for nexttag in tag_iter:
# #                 tagname = nexttag.tag
# #                 if tagname == 'head' and nexttag.tag_type == HtmlTagType.CLOSE_TAG or tagname == 'body':
# #                     break
# #                 if tagname == 'base':
# #                     href = nexttag.attributes.get('href')
# #                     if href:
# #                         joined_base = urljoin(htmlpage.url, href.strip().strip(), htmlpage.encoding)
# #                         base_href = remove_entities(joined_base, encoding=htmlpage.encoding)
# #                 elif tagname == 'meta':
# #                     attrs = nexttag.attributes
# #                     if attrs.get('http-equiv') == 'refresh':
# #                         m = _META_REFRESH_CONTENT_RE.search(attrs.get('content') or '')
# #                         if m:
# #                             target = m.group('url')
# #                             if target:
# #                                 yield mklink(target)
# #                 elif tagname == 'link':
# #                     href = nexttag.attributes.get('href')
# #                     if href:
# #                         yield mklink(href)
# #         elif tagname == 'area':
# #             href = attributes.get('href')
# #             if href:
# #                 nofollow = attributes.get('rel') == 'nofollow'
# #                 yield mklink(href, attributes.get('alt', ''), nofollow)
# #         elif tagname in ('frame', 'iframe'):
# #             target = attributes.get('src')
# #             if target:
# #                 yield mklink(target)
# #         elif 'onclick' in attributes:
# #             match = _ONCLICK_LINK_RE.search(attributes['onclick'] or '')
# #             if not match:
# #                 continue
# #             target = match.group('url')
# #             nofollow = attributes.get('rel') == u'nofollow'
# #             yield mklink(target, nofollow=nofollow)
# #     if astart:
# #         yield mklink(ahref, htmlpage.body[astart:])
# '''
#     #     code1 = '''
#     # for i in range(2):
#     #     a=1
#     #     b=2
#     #     for j in range(3):
#     #         w=8
#     #         break
#     #     if i>1 and j>3 and i!=0:
#     #         if i>2 and call(1,t,*T):
#     #             b=3
#     #             break
#     #     c=3
#     #     '''
#
#     tree = ast.parse(code)
#     # for node in ast.walk(tree):
#     #     if isinstance(node,(ast.For,ast.While)):
#     #         print("-------------------------")
#     #         assign_list_in_for,if_list_in_for=[],[]
#     #         whether_contain_break_and_const_assign(node, assign_list_in_for, if_list_in_for)
#     #         print(assign_list_in_for,if_list_in_for)
#     #         # assign_list_in_for=[]
#     #         # if_list_in_for=[]
#     #         # break_flag=whether_contain_break_and_const_assign(node, assign_list_in_for,if_list_in_for)
#     #         #
#     #         # print(break_flag,assign_list_in_for,if_list_in_for)
#     #     elif isinstance(node,ast.If):
#     #         if_vars=[]
#     #         visit_vars(node.test,if_vars)
#     #         print("if vars: ",if_vars)
#     # code_list = []
#     # ass_init_list = []
#     # traverse_cur_layer(tree, code_list, ass_init_list)
#     # print("*****************: ", code_list)
#     # code_list=[]
#     # tree=ast.parse(code1)
#     # traverse_cur_layer(tree,code_list)
#     # print(code_list)
#
#     # for_else_filter_redundant_code_list = get_for_else_complicate_code(file_tree)
#     # print(for_else_filter_redundant_code_list)
#     # for one_code in for_else_filter_redundant_code_list:
#     #     print("-----------------------",one_code)
#     #     print(one_code[0])
#
#     # file_tree = ast.parse(code1)
#     # ana_py = Analyzer()
#     # ana_py.visit(file_tree)
#     # print("fun number: ",len(ana_py.func_def_list))
#
#     # save_complicated_code_dir= util.data_root + "complicated_code_dir_1000_star/for_else_assign_is_const_complicated/"
#     # save_complicated_code_dir = util.data_root + "transform_complicate_to_simple/for_else_improve_5/"
#     save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl_improve_new_modify_remove_flag/for_else/"
#     save_complicated_code_dir_pkl = util.data_root + "transform_complicate_to_simple_pkl_improve_new_modify_bugs_final/for_else/"
#
#     dict_repo_file_python= util.load_json(util.data_root, "python3_1000repos_files_info")
#
#     # dict_repo_file_python = util.load_json(util.data_root, "jupyter3_repos_files_info")
#
#     repo_list = []
#     count_file = 0
#     for ind, repo_name in enumerate(dict_repo_file_python):
#         # if repo_name!="ansible-modules-core":#"DeepDanbooru":#"python-samples":#"sympy":
#         #     continue
#
#         # if os.path.exists(save_complicated_code_dir_pkl + repo_name + ".pkl"):
#         #     continue
#         # if os.path.exists(save_complicated_code_dir + repo_name + ".json"):
#         #     continuetransform_for_else_compli_to_simple_improve.py
#         # if ind>30:
#         #     break
#         # print("repo_name: ", repo_name)
#         repo_list.append(repo_name)
#         # break
#     print("len: ", len(repo_list))
#     # save_repo_for_else_complicated(repo_list[0])
#     # savetransform_c_s_repo_for_else_complicated(repo_list[0])
#     '''
#         for file_info in dict_repo_file_python[repo_name]:
#
#             file_path = file_info["file_path"]
#             # if file_path!="/mnt/zejun/smp/data/python_repo_1000/VideoPose3D//run.py":
#             #     continue
#             file_html = file_info["file_html"]
#             #print("come this file: ", file_path)
#             try:
#                 content = util.load_file(file_path)
#             except:
#                 print(f"{file_path} is not existed!")
#                 continue
#
#             #print("content: ",content)
#             try:
#                 file_tree = ast.parse(content)
#                 count_file+=1
#             except:
#                 continue
#     print("number of repos: ", len(repo_list),count_file)
#     '''
#     '''
#     count_complicated_code=0
#     for ind,repo_name in enumerate(dict_repo_file_python):
#         print("come repo: ",repo_name)
#         count_complicated_code+=save_repo_for_else_complicated(repo_name)
#         # break
#     print("count_complicated_code: ",ind,count_complicated_code)
#     '''
#     # for repo in repo_list:
#     #     # if repo!="django-extensions":#"keras-bert":#"sympy":#"Real-Time-Voice-Cloning":#"keras-bert":#"Legofy":
#     #     #     continue
#     #     save_repo_for_else_complicated(repo)
#
#     '''
#     #7.8h +
#     pool = newPool(nodes=30)
#     pool.map(save_repo_for_else_complicated, repo_list)  # [:3]sample_repo_url ,token_num_list[:1]
#     pool.close()
#     pool.join()
#     # print("len all_files: ", len(all_files))
#     '''
#
#
#     # repo_files_info = util.load_json(util.data_root, "dict_1000_repo_files_info")
#     # repo_star_info = util.load_json(util.data_root, "dict_1000_repo_star_info")
#     # repo_contributor_info = util.load_json(util.data_root, "dict_1000_repo_contributor_info")
#
#
#     def get_code_count(complicate_code):
#         all_me_count=0
#         all_file_count=len(list(complicate_code.keys()))
#         code_count = 0
#         me_count=0
#         NULL_code_count=0
#         file_count = 0
#         cl_count=0
#         for file_html in complicate_code:
#             if complicate_code[file_html]:
#                 file_exist=0
#                 # cl_count +=len(list(complicate_code[file_html].keys()))
#                 for cl in complicate_code[file_html]:
#                     if complicate_code[file_html][cl]:
#                         all_me_count+=len(complicate_code[file_html][cl].keys())
#                         for me in complicate_code[file_html][cl]:
#                             if complicate_code[file_html][cl][me]:
#                                 me_count += 1
#                                 file_exist=1
#                                 code_count += len(complicate_code[file_html][cl][me])
#
#                 file_count += file_exist
#
#         return file_count,me_count,code_count,all_file_count,all_me_count
#
#
#     files_num_list = []
#     star_num_list = []
#     contributor_num_list = []
#     count_repo, file_count, me_count, code_count = 0, 0, 0, 0
#     file_list = set([])
#     repo_code_num = dict()
#     result_compli_for_else_list = []
#     all_count_repo, all_file_count, all_me_count = 0, 0, 0
#     for file_name in os.listdir(save_complicated_code_dir_pkl):
#         all_count_repo+=1
#         repo_name = file_name[:-4]
#         # if repo_name!="sympy":
#         #     continue
#         # files_num_list.append(repo_files_info[repo_name])
#         # star_num_list.append(repo_star_info[repo_name])
#         # contributor_num_list.append(repo_contributor_info[repo_name])
#
#         complicate_code = util.load_pkl(save_complicated_code_dir_pkl, repo_name)
#
#         repo_file_count,repo_me_count,repo_code_count,repo_all_file_count,repo_all_me_count= complicated_code_util.get_code_count(complicate_code)
#         # for code_list, file_path, file_html in complicate_code:
#         code_count+=repo_code_count
#         file_count+=repo_file_count
#         me_count+=repo_me_count
#         all_file_count+=repo_all_file_count
#         all_me_count+=repo_all_me_count
#         repo_exist=0
#         for file_html in complicate_code:
#              for cl in complicate_code[file_html]:
#                 for me in complicate_code[file_html][cl]:
#                     if complicate_code[file_html][cl][me]:
#                         repo_exist=1
#                         for code in complicate_code[file_html][cl][me]:
#
#                             # print("html: ",file_html,cl,me,ast.unparse(code1[0]))
#                                 #code_list.append([tree,tree_copy,break_list_in_for,child,child_copy,intersect_infor_ass_init[0][1],init_ass_remove_flag])
#                                 pass
#                                 result_compli_for_else_list.append(
#                                 [repo_name, file_html, cl,me,code[3].lineno,ast.unparse(code[5])+"\n"+ast.unparse(code[3])+"\n"+ast.unparse(code[6]), ast.unparse(code[4]),code[7]])
#
#             # print(f"{file_html} of {repo_name} has  {len(code_list)} code1 fragments")
#         count_repo += repo_exist
#             #     break
#             # break
#         # if complicate_code:
#         #     repo_code_num[repo_name]=count_code_fra
#         #     print(f"{repo_name} has  {count_code_fra} code1 fragments")
#         # break
#     # a=dict(sorted(repo_code_num.items(), key=lambda item: item[1], reverse=True))
#     # print(a)
#     # print(np.median(list(a.values())), np.max(list(a.values())), np.min(list(a.values())))
#     # print(np.median(files_num_list), np.max(files_num_list), np.min(files_num_list))
#     # print(np.median(star_num_list), np.max(star_num_list), np.min(star_num_list))
#     # print(np.median(contributor_num_list), np.max(contributor_num_list), np.min(contributor_num_list))
#     print("count: ", len(result_compli_for_else_list),count_repo,code_count,file_count,me_count,all_count_repo,all_file_count,all_me_count)
#     #1 156 2943 100 1 2990 40102 salt
#     #791 2010 103165 1291 800 121348 1192868
#     import random
#
#     # random.shuffle(result_compli_for_else_list)
#     # util.save_csv(util.data_root + "result_csv/for_else_improve_new_modify_bugs_final.csv",
#     #               result_compli_for_else_list[:400],
#     #               ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code"])
#
#     # util.save_csv(util.data_root + "result_csv/for_else_improve_modify_remove_flag_2.csv",
#     #               result_compli_for_else_list[:400],
#     #               ["repo_name", "file_html", "class_name", "me_name", "line_no", "old_code", "new_code"])
#
#     # util.save_csv(util.data_root+"complicated_code_dir_1000_star/for_else_iftest_intersect_iftest_assign.csv",result_compli_for_else_list,["repo_name","code1","file_html","file_path"])
#
#     # util.save_csv(util.data_root + "complicated_jupyter_code_dir/for_else.csv", result_compli_for_else_list,
#     #               ["repo_name", "code1", "file_html", "file_path"])
#
#     # print("----------------------------\n")
#     # for code1 in for_else_filter_redundant_code_list:
#     #     print("each code1: ", code1[0])
# #https://github.com/KichangKim/DeepDanbooru/tree/master/deepdanbooru/commands/download_tags.py 80
# #https://github.com/ansible/ansible-modules-core/tree/master/cloud/openstack/os_router.py 84



import ast
Keywords=["False",      "await",      "else",       "import",     "pass",
"None",       "break",      "except",     "in",         "raise",
"True",       "class",      "finally",    "is",         "return",
"and",        "continue",   "for",        "lambda",     "try",
"as",         "def",        "from",       "nonlocal",   "while",
"assert",     "del",       "global",     "not",        "with",
"async",      "elif",       "if",         "or",         "yield"]
class Fun_Analyzer(ast.NodeVisitor):
    def __init__(self,class_name=""):
        self.func_def_list = []
        self.class_name = class_name
    def visit_FunctionDef(self, node):
        self.func_def_list.append([node, self.class_name])
        ast.NodeVisitor.generic_visit(self, node)
    def generic_visit(self, node):
        ast.NodeVisitor.generic_visit(self, node)

    def visit_ClassDef(self, node):

        class_ana=Fun_Analyzer(node.name)
        for stmt in node.body:
            class_ana.visit(stmt)
        self.func_def_list.extend(class_ana.func_def_list)
    def visit_If(self, node: ast.If):
        if ast.unparse(node.test) == "__name__ == '__main__'":
            self.func_def_list.append([node,""])
class Analyzer(ast.NodeVisitor):
    def __init__(self):
        self.func_def_list = []

    def visit_FunctionDef(self,node):
        self.func_def_list.append(node)
    def visit_If(self, node: ast.If):
        if ast.unparse(node.test)=="__name__ == '__main__'":
            self.func_def_list.append(node)
def set_dict_class_code_list(tree,dict_class,class_name,new_code_list):
    me_name = tree.name if hasattr(tree, "name") else "if_main_my"
    me_lineno = tree.lineno
    me_id = "".join([me_name, "$", str(me_lineno)])

    if class_name not in dict_class:
        dict_class[class_name] = dict()
        dict_class[class_name][me_id] = new_code_list
    else:

        dict_class[class_name][me_id] = new_code_list

def get_basic_count(e):

    count=0
    # print("e dict: ",e.__dict__)
    if isinstance(e, (ast.Tuple,ast.List)):
        # count += len(e.elts)
        for cur in e.elts:
            count +=get_basic_count(cur)

    else:
        # print(e.__dict__, " are not been parsed")
        count +=1


    return count
def get_basic_object(e,var_list=[]):
    if isinstance(e, (ast.Tuple,ast.List)):
        # count += len(e.elts)
        for cur in e.elts:
            get_basic_object(cur,var_list)

    else:
        # print(e.__dict__, " are not been parsed")
        var_list.append(ast.unparse(e))


def extract_ast_block_node(node,node_list):
    for k in node._fields:

        v = getattr(node, k)

        if isinstance(v, list):
            # print("come here", v)
            a=[]
            for e in v:
                a.append(e)
                if isinstance(e, ast.AST):
                    extract_ast_block_node(e, node_list)
            node_list.append(a)
        elif isinstance(v, ast.AST):
            # print("come here", v.__dict__)
            if v._fields:
                extract_ast_block_node(v,node_list)
def extract_ast_cur_layer_node(node,node_list):
    # if node._fields:
    #     node_list.append(list(node._fields))
    for k in node._fields:
        v = getattr(node, k)

        if isinstance(v, list):
            a = []
            for e in v:
                a.append(e)
            node_list.append(a)
            for e in v:
                if e._fields:
                    extract_ast_block_node(e, node_list)


        elif isinstance(v, ast.AST):
            if v._fields:
                extract_ast_block_node(v, node_list)

if __name__ == '__main__':
    code='''
a.b=2
if label_names:
    a=2
    num_tensors_in_label = len(label_names)
else:
    num_tensors_in_label = int(has_labels)
# a=1
# b=c.b+1
a=1
b=2
c,d=1,2
for i in range(2):
    a=1
    if i>1:
        if i>2:
            a=3
        a=2
        break 
if a==2:
    print(1)
print("1")   
def a():
    return 1
for i in range(5):
    a=1
    break
if a:
    print("test")
if __name__ == '__main__':
    a=1
    
'''
    tree = ast.parse(code)
    node_list=[]
    extract_ast_cur_layer_node(tree, node_list)
    for e in node_list:
        print(e)
